#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from .control_widget_pytemplate import *
import psutil
import os
from ....common.utils import make_workdir, make_runfile
from ....common.qdialogs import *
from ....common import console_history_path
import pickle
from ....analysis import Transmission, organize_dataframe_columns
import numpy as np
from functools import partial
from collections import deque
from ....pyqtgraphCore.widgets.MatplotlibWidget import MatplotlibWidget
from ....pyqtgraphCore.console import ConsoleWidget
from itertools import product as iter_product
from seaborn import lineplot
from ..proportions import ProportionsWidget
from ..base import BasePlotWidget
from math import sqrt, ceil
from ...utils import auto_colormap
from typing import Union
from ...variants import Heatmap
import json
import pandas as pd
from copy import deepcopy

from ....common.configuration import HAS_TSLEARN, IS_WINDOWS

if HAS_TSLEARN:
    from tslearn.preprocessing import TimeSeriesScalerMeanVariance
    from tslearn.clustering import KShape
    from . import kshape_process

if not IS_WINDOWS:
    from signal import SIGKILL

elif IS_WINDOWS:
    from win32api import TerminateProcess, CloseHandle


class KShapeControlDock(QtWidgets.QDockWidget):
    def __init__(self, parent):
        QtWidgets.QDockWidget.__init__(self, parent=parent)
        self.ui = Ui_KShapeControl()
        self.ui.setupUi(self)
        self.setFloating(False)

        self.ui.checkBoxUseGridsearch.clicked.connect(
            partial(
                self.set_checkbox_use_grid_or_single,
                self.ui.checkBoxUseGridsearch
            )
        )

        self.ui.checkBoxUseSingle.clicked.connect(
            partial(
                self.set_checkbox_use_grid_or_single,
                self.ui.checkBoxUseSingle
            )
        )

    def set_checkbox_use_grid_or_single(self, w: QtWidgets.QCheckBox):
        if w is self.ui.checkBoxUseGridsearch:
            self.ui.checkBoxUseSingle.setChecked(False)
        elif w is self.ui.checkBoxUseSingle:
            self.ui.checkBoxUseGridsearch.setChecked(False)

    def get_params(self) -> dict:
        if self.ui.checkBoxRandom.isChecked():
            random_state = None
        else:
            random_state = self.ui.spinBoxRandom.value()

        if self.ui.checkBoxUseGridsearch.isChecked():
            return\
                {
                    'npartitions_range': (self.ui.spinBoxNPartMin.value(), self.ui.spinBoxNPartMax.value()),
                    'ncombinations': 10 ** self.ui.spinBoxNCombinations.value(),
                    'sortby': self.ui.comboBoxSortBy.currentText(),
                    'max_iter': self.ui.spinBoxMaxIter.value(),
                    'tol': 10 ** self.ui.spinBoxTol.value(),
                    'n_init': self.ui.spinBoxN_init.value(),
                    'train_percent': self.ui.spinBoxTrainSubsetPercentage.value()
                }

        elif self.ui.checkBoxUseSingle.isChecked():
            return \
                {
                    'n_clusters': self.ui.spinBoxN_clusters.value(),
                    'max_iter': self.ui.spinBoxMaxIter.value(),
                    'tol': 10 ** self.ui.spinBoxTol.value(),
                    'n_init': self.ui.spinBoxN_init.value(),
                    'random_state': random_state,
                    'train_percent': self.ui.spinBoxTrainSubsetPercentage.value()
                }
        else:
            raise ValueError(
                'Must select one of "Use Gridsearch" or "Use Single" in kShape params.'
            )

    def set_active(self):
        self.ui.pushButtonStart.setDisabled(True)
        self.ui.pushButtonAbort.setEnabled(True)

        self.ui.groupBoxKShapeParams.setEnabled(True)

    def set_inactive(self):
        self.ui.pushButtonStart.setEnabled(True)
        self.ui.pushButtonAbort.setDisabled(True)
        self.ui.groupBoxKShapeParams.setDisabled(False)


class KShapePlot(QtWidgets.QDockWidget):
    """Curves plots, showing a sample of individual curves from a single cluster"""
    def __init__(self, parent):
        """"""
        QtWidgets.QDockWidget.__init__(self, parent=parent)
        self.plot = MatplotlibWidget()  #: MatplotlibWidget() instance
        self.setWidget(self.plot)

        self.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable | QtWidgets.QDockWidget.DockWidgetMovable)
        self.setMinimumSize(QtCore.QSize(300, 300))

        self.fig = self.plot.fig
        self.ax = self.fig.add_subplot(111)  #: The Axes object for this plot
        self.draw = self.plot.draw
        self.setWindowTitle('Samples of raw curves in a cluster')


class KShapeMeansPlot(MatplotlibWidget):
    """Means plots grouped by cluster membership"""
    def __init__(self, parent):
        # QtWidgets.QDockWidget.__init__(self, parent=parent)
        MatplotlibWidget.__init__(self)
        self.axs = None  #: array of axis objects used for drawing the means plots, shape is [nrows, ncols]
        self.setParent(parent)
        self.ncols = 3
        self.nrows = None

    def set_plots(self, input_arrays: np.ndarray, n_clusters: int, y_pred: np.ndarray, xzero_pos: str, error_band):
        """
        Set the subplots

        :param input_arrays: padded input arrays (2D),  shape is [num_samples, padded_peak_curve_length]
        :param n_clusters: number of clusters
        :param y_pred: cluster predictions (labels)
        :param xzero_pos: set the zero position as the 'zero' position of
                          the input array or the 'maxima' of the input array
        :param error_band: Type of error band to show, one of either 'ci' or 'std'
        """
        # nrows, ncols = (int(ceil(sqrt(n_clusters))), int(sqrt(n_clusters)))

        self.nrows = ceil(n_clusters / self.ncols)

        if n_clusters < 11:
            cmap = 'tab10'
        elif 10 < n_clusters < 21:
            cmap = 'tab20'
        elif 20 < n_clusters < 211:
            cmap = 'nipy_spectral'
        else:
            raise ValueError("Cannot generate colormap for greater than 210 clusters.\n"
                             "What are you trying to do with > 210 clusters, what's wrong with you?")

        colors = auto_colormap(n_clusters, cmap, 'mpl', 'subsequent')

        self.fig.clear()
        self.axs = self.fig.subplots(self.nrows, self.ncols)
        self.fig.tight_layout()

        for c_ix, i in enumerate(iter_product(range(self.nrows), range(self.ncols))):
            if self.nrows == 1:  # matplotlib just uses a 1D axes array if nrows=1
                i = i[1]

            if c_ix == n_clusters:
                break

            self.axs[i].set_title(f"cluster {c_ix}")

            ys = input_arrays[y_pred == c_ix]

            if ys.size == 0:
                self.axs[i].set_title(f"cluster {c_ix}, EMPTY")
                continue

            xs = []

            for y in ys:
                if xzero_pos == 'zero':
                    zero_ix = 0
                elif xzero_pos == 'maxima':
                    zero_ix = np.argmax(y)
                else:
                    raise ValueError('xzer_post argument accepts only either "zero" or "maxima"')
                xs.append(np.arange((0 - zero_ix), y.size - zero_ix))

            xsh = np.hstack(xs)
            ysh = np.hstack(ys)

            lineplot(x=xsh, y=ysh, ax=self.axs[i], err_style='band', color=colors[c_ix], ci=error_band)

        self.draw()


class ProportionsDock(QtWidgets.QDockWidget):
    def __init__(self, parent):
        QtWidgets.QDockWidget.__init__(self, parent=parent)

        self.plot = ProportionsWidget()
        self.setWidget(self.plot)

        self.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable | QtWidgets.QDockWidget.DockWidgetMovable)
        self.setMinimumSize(QtCore.QSize(100, 600))
        self.setWindowTitle('Proportions')


class KShapeWidget(QtWidgets.QMainWindow, BasePlotWidget):
    """User-end KShape widget"""
    sig_output_changed = QtCore.pyqtSignal(Transmission)  #: Emits output Transmission containing cluster labels
    drop_opts = None  #: Unused by this plot widget

    def __init__(self, parent=None):
        QtWidgets.QMainWindow.__init__(self, parent=parent)
        BasePlotWidget.__init__(self)
        self.setWindowTitle('k-Shape Clustering')

        self.control_widget = KShapeControlDock(parent=self)
        self.addDockWidget(QtCore.Qt.LeftDockWidgetArea, self.control_widget)

        self.proportions_widget = ProportionsDock(self)
        self.addDockWidget(QtCore.Qt.RightDockWidgetArea, self.proportions_widget)

        self.plot_proportions = self.proportions_widget.plot  #: Embedded Proportions Plot

        self.finished = False
        self.process = None  #: External QProcess that runs the clustering

        self._workdir = None

        self._input_arrays = None
        self._params = None
        self._n_clusters = None
        self._ks = None
        self._train_data = None
        self._y_pred = None
        self._cluster_centers = None
        self._cluster_means = None

        self.data_column = None
        self.input_connected = True

        self.train_percentage = None

        self.std_out = deque(maxlen=500)

        self.control_widget.ui.pushButtonStart.clicked.connect(self.start_process)
        self.control_widget.ui.pushButtonAbort.clicked.connect(self.abort_process)

        self.control_widget.ui.listWidgetClusterNumber.currentItemChanged.connect(self.update_plot)

        self.control_widget.ui.pushButtonApplyPlotOptions.clicked.connect(self.update_plot)
        self.control_widget.ui.pushButtonApplyPlotOptions.clicked.connect(self.update_plot_means)

        self.control_widget.ui.pushButtonSave.clicked.connect(self.save_plot_dialog)
        self.control_widget.ui.pushButtonLoad.clicked.connect(self.open_plot_dialog)

        self.control_widget.ui.pushButtonReconnectFlowchartInput.clicked.connect(self.input_connect)
        self.control_widget.ui.pushButtonReconnectFlowchartInput.setVisible(False)

        self.plot_raw = KShapePlot(parent=self)  #: The single curve plots

        self.plot_means = KShapeMeansPlot(parent=self)  #: The means plots

        self.setCentralWidget(self.plot_means)

        self.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.plot_raw)
        self.dockConsole = QtWidgets.QDockWidget(self)
        self.dockConsole.setWindowTitle('Console')
        self.dockConsole.setFeatures(
            QtWidgets.QDockWidget.DockWidgetFloatable | QtWidgets.QDockWidget.DockWidgetMovable)

        cmd_history_file = os.path.join(console_history_path, 'kshape.pik')

        ns = {'this': self,
              'get_plot_means': lambda: self.plot_means,
              'get_plot_raw': lambda: self.plot_raw,
              'get_plot_proportions': lambda: self.plot_proportions
              }

        txt = ["Namespaces",
               "self as 'this'",
               "Useful callables: get_plot_means(), get_plot_raw(), get_plot_proportions()",
               "Get the clusters with lowest inertia: this.inertia_sorted.head()"
               ]

        txt = "\n".join(txt)

        self.dockConsole.setWidget(ConsoleWidget(parent=self, namespace=ns, text=txt, historyFile=cmd_history_file))
        self.addDockWidget(QtCore.Qt.BottomDockWidgetArea, self.dockConsole)

        self._ksgrid = None
        self.__ksgrid_json = None
        self.kga_inertia_heatmap: Heatmap = Heatmap(highlight_mode='item')
        self.kga_inertia_heatmap.hide()
        self.kga_inertia_heatmap.sig_selection_changed.connect(self.update_ksgrid_selection)
        self.kga_inertia_heatmap_cmap = 'jet_r'
        self.inertia_sorted: pd.DataFrame = None

        self.n_clusters_out: np.ndarray = np.empty(0)

        self.resize(1500, 900)

    def set_update_live(self, b: bool):
        """Not used in this widget"""
        pass

    @property
    def input_arrays(self) -> np.ndarray:
        """
        The input arrays for clustering

        :return: 2D array, shape is [num_samples, padded_peak_curve_length]
        :rtype: np.ndarray
        """

        if self._input_arrays is None:
            raise ValueError('Input array not set')

        return self._input_arrays

    @input_arrays.setter
    def input_arrays(self, a: Union[np.ndarray, list]):
        """
        Set the input arrays

        :param a: 1D array of all input arrays, shape is (num_samples, )
        """

        if isinstance(a, list):
            a = np.array(a)

        if not isinstance(a, np.ndarray):
            raise TypeError('Must pass np.ndarray or list')

        elif a.ndim > 2:
            raise ValueError('Array must be 2D')

        self._input_arrays = a

    @property
    def ks(self) -> KShape:
        """
        tslearn KShape object
        """

        if self._ks is None:
            raise ValueError('KShape not instantiated')

        return self._ks

    @ks.setter
    def ks(self, ks):

        if not isinstance(ks, kshape_process.KShape):
            raise TypeError('Must pass KShape instance')

        self._ks = ks

    @property
    def ksgrid(self) -> np.ndarray:
        if self._ksgrid is None:
            raise AttributeError('`ksgrid` not set')
        else:
            return self._ksgrid

    @ksgrid.setter
    def ksgrid(self, ksgrid: np.ndarray):
        self._ksgrid = ksgrid

        p_range = self.params['kwargs']['npartitions_range']

        kga_inertia = np.zeros(self.ksgrid.shape, dtype=np.float64)
        for ij in iter_product(range(self.ksgrid.shape[0]), range(self.ksgrid.shape[1])):
            if self.n_clusters_out[ij] < self.ksgrid[ij].n_clusters:
                kga_inertia[ij] = np.nan
            else:
                kga_inertia[ij] = self.ksgrid[ij].inertia_

        self.kga_inertia_heatmap.set(
            kga_inertia,
            ylabels=list(range(*p_range)),
            cmap=self.kga_inertia_heatmap_cmap,
            annot=False
        )

        ixs_lowest = np.dstack(np.unravel_index(np.argsort(kga_inertia.ravel()), kga_inertia.shape))[0]
        ixs_lowest[:, 0] += self.params['kwargs']['npartitions_range'][0]

        self.inertia_sorted = pd.DataFrame.from_dict(
            {
                'n_clusters':   ixs_lowest[:, 0],
                'trial':        ixs_lowest[:, 1]
            }
        )

        self.kga_inertia_heatmap.show()

    def _lists_to_arrays(self, model):
        for k in model['model_params'].keys():
            param = model['model_params'][k]
            if type(param) is list:
                arr = np.array(param)
                if arr.dtype == object:
                    # Then maybe it was rather a list of arrays
                    # This is very hacky...
                    arr = [np.array(p) for p in param]
                model['model_params'][k] = arr

    def _create_model(self, model):
        """
        A messy function until the latest tslearn is out
        with a fix for array-based hyper-parameters
        """
        # Convert the lists back to arrays
        for k in model['model_params'].keys():
            param = model['model_params'][k]
            if type(param) is list:
                arr = np.array(param)
                if arr.dtype == object:
                    # Then maybe it was rather a list of arrays
                    # This is very hacky...
                    arr = [np.array(p) for p in param]
                model['model_params'][k] = arr

        for k in model['hyper_params'].keys():
            param = model['hyper_params'][k]
            if type(param) is list:
                arr = np.array(param)
                if arr.dtype == object:
                    # Then maybe it was rather a list of arrays
                    # This is very hacky...
                    arr = [np.array(p) for p in param]
                model['hyper_params'][k] = arr
        hyper_params = model['hyper_params']
        model_params = model['model_params']

        inst = KShape(**hyper_params)

        for p in model_params.keys():
            setattr(inst, p, model_params[p])

        return inst

    @property
    def _ksgrid_json(self):
        if self.__ksgrid_json is None:
            raise AttributeError('`__ksgrid_json` not set')
        return self.__ksgrid_json

    @_ksgrid_json.setter
    def _ksgrid_json(self, ksgrid_json: dict):
        self.__ksgrid_json = deepcopy(ksgrid_json)
        # create the models from the json strings
        kgl = [self._create_model(model) for model in ksgrid_json]

        kga = np.array(kgl, dtype=object).reshape(
            len(range(*self.params['kwargs']['npartitions_range'])),  # same shape as the nested loop
            self.params['kwargs']['ncombinations'],
        )

        self.ksgrid = kga

    @property
    def n_clusters(self) -> int:
        """
        Number of clusters

        :rtype: int
        """

        if self._n_clusters is None:
            raise ValueError('Must finish clustering first')

        return self._n_clusters

    @n_clusters.setter
    def n_clusters(self, n: int):

        if type(n) is not int:
            raise TypeError('n_clusters must be int')

        self._n_clusters = n

    @property
    def train_data(self) -> np.ndarray:
        """
        The training data for clustering

        :return: Training data as a 2D array, shape is [n_samples, padded_curve_length]
        :rtype: np.ndarray
        """

        if self._train_data is None:
            raise ValueError('Must run clustering first')

        return self._train_data

    @train_data.setter
    def train_data(self, data: Union[np.ndarray, list]):

        if isinstance(data, list):
            data = np.array(data)

        if not isinstance(data, np.ndarray):
            raise ValueError('Must pass numpy array or list')

        self._train_data = data

    @property
    def y_pred(self) -> np.ndarray:
        """
        Predicted cluster labels after the model converges

        :return: 1D array of cluster labels that correspond to the input_data
        :rtype: np.ndarray
        """

        if self._y_pred is None:
            raise ValueError('No predictions have been fit')

        return self._y_pred

    @y_pred.setter
    def y_pred(self, y_pred: Union[np.ndarray, list]):
        """
        Predictions of cluster members, i.e. cluster labels

        :param y_pred:
        """

        if isinstance(y_pred, list):
            y_pred = np.array(y_pred)

        if not isinstance(y_pred, np.ndarray):
            raise TypeError('Must pass numpy array or list')

        self._y_pred = y_pred

    @property
    def cluster_centers(self) -> np.ndarray:
        """
        Cluster centroids

        :return: 2D array, shape is [n_clusters, centroid_array]
        :rtype: np.ndarray
        """

        if self._cluster_centers is None:
            raise ValueError('Must finish clustering first')

        return self._cluster_centers

    @cluster_centers.setter
    def cluster_centers(self, c: Union[np.ndarray, list]):

        if isinstance(c, list):
            c = np.array(c)

        if not isinstance(c, np.ndarray):
            raise TypeError('Must pass numpy array or list')

        self._cluster_centers = c

    @property
    def cluster_means(self) -> np.ndarray:
        """
        The cluster means

        :return: 2D array, shape is [cluster_label, mean_array]
        :rtype: np.ndarray
        """

        if self._cluster_means is None:
            raise ValueError('No predictions have been fit')

        return self._cluster_centers

    @cluster_means.setter
    def cluster_means(self, cms: Union[np.ndarray, list]):

        if isinstance(cms, list):
            cms = np.array(cms)

        if not isinstance(cms, np.ndarray):
            raise TypeError('Must pass np.ndarray or list')

        self._cluster_means = cms

    @property
    def params(self) -> dict:
        """
        Parameters dict.

        :rtype: dict
        """

        if self._params is None:
            raise ValueError('Params not set')

        return self._params

    @params.setter
    def params(self, d: dict):
        self._params = d

    def clear_data(self):
        self._ks = None
        self._y_pred = None
        self._input_arrays = None

    def set_input(self, transmission: Transmission):
        """
        Set the input Transmission for the widget

        :param transmission: Input Transmission
        """
        if not self.input_connected:
            return
        super(KShapeWidget, self).set_input(transmission)
        self.transmission.df.reset_index(drop=True, inplace=True)
        self.proportions_widget.plot.set_input(self.transmission)

    def fill_control_widget(self, data_columns: list, categorical_columns: list, uuid_columns: list):
        self.control_widget.ui.comboBoxDataColumn.clear()
        self.control_widget.ui.comboBoxDataColumn.addItems(data_columns)

    def pad_input_data(self, a: np.ndarray, method: str = 'random') -> np.ndarray:
        """
        Pad all the input arrays so that are of the same length. The length is determined by the largest input array.
        The padding value for each input array is the minimum value in that array.

        Padding for each input array is either done after the array's last index to fill up to the length of the
        largest input array (method 'fill-size') or the padding is randomly flanked to the input array (method 'random)
        for easier visualization.

        :param a: 1D array of input arrays where each element is a sample array
        :param method: 'fill-size' or 'random'
        :return: 2D array of the padded arrays in the rows
        """
        l = 0  # size of largest time series

        # Get size of largest time series
        for c in a:
            s = c.size
            if s > l:
                l = s

        # pre-allocate output array
        p = np.zeros(shape=(a.size, l))

        # pad each 1D time series
        for i in range(p.shape[0]):
            s = a[i].size

            if s == l:
                p[i, :] = a[i]
                continue

            max_pad_en_ix = l - s

            if method == 'random':
                pre = np.random.randint(0, max_pad_en_ix)
            elif method == 'fill-size':
                pre = 0
            else:
                raise ValueError('Must specific method as either "random" or "fill-size"')

            post = l - (pre + s)
            p[i, :] = np.pad(a[i], (pre, post), 'minimum')

        return p

    @present_exceptions('Start process error', 'Make sure you have selected an appropriate Data Column')
    def start_process(self, *args, **kwargs):
        """Start the the KShape clustering in a QProcess"""
        if self.finished:
            if QtWidgets.QMessageBox.warning(self, 'Discard data?',
                                             'Would you like to discard the current clustering data?',
                                             QtWidgets.QMessageBox.Yes,
                                             QtWidgets.QMessageBox.No) == QtWidgets.QMessageBox.No:
                return
            self.plot_raw.ax.cla()

        self.finished = False
        self.input_disconnect()
        self.clear_data()

        self.data_column = self.control_widget.ui.comboBoxDataColumn.currentText()
        self.input_arrays = self.transmission.df[self.data_column].values

        self.train_percentage = self.control_widget.ui.spinBoxTrainSubsetPercentage.value()

        padded = self.pad_input_data(self.input_arrays, method='fill-size')
        # scaler = TimeSeriesScalerMeanVariance()
        # self.input_arrays = scaler.fit_transform(padded)[:, :, 0]

        workdir = self.get_workdir()

        self.clear_workdir()

        self.params = \
            {
                'kwargs': self.control_widget.get_params(),
                'workdir': workdir,
                'out': os.path.join(workdir, 'out'),
                'do_single': self.control_widget.ui.checkBoxUseSingle.isChecked(),
                'do_grid': self.control_widget.ui.checkBoxUseGridsearch.isChecked()
            }

        params_path = os.path.abspath(workdir + '/params.pickle')
        pickle.dump(self.params, open(params_path, 'wb'))

        data_path = os.path.abspath(workdir + '/data.npy')
        np.save(data_path, padded)

        self.process = QtCore.QProcess()
        self.process.setWorkingDirectory(self.get_workdir())

        self.process.setProcessChannelMode(QtCore.QProcess.MergedChannels)
        self.process.readyReadStandardOutput.connect(partial(self.print_stdout, self.process))
        self.process.readyReadStandardError.connect(partial(self.print_stdout, self.process))
        self.process.finished.connect(self.process_finished)

        m = os.path.abspath(kshape_process.__file__)
        args = data_path + ' ' + params_path
        sh_file_path = make_runfile(module_path=m, savedir=self.get_workdir(), args_str=args)

        self.control_widget.ui.textBrowser.clear()
        
        if IS_WINDOWS:
            self.process.start('powershell.exe', [sh_file_path])
        else:
            self.process.start(sh_file_path)

        self.process.start(sh_file_path)

        self.control_widget.set_active()

    def print_stdout(self, process: QtCore.QProcess):
        std_out = process.readAll().data().decode('utf8')
        self.control_widget.ui.textBrowser.append(std_out)

    def get_workdir(self):
        """Get the workdir"""
        if self._workdir is None:
            self._workdir = make_workdir('kshape')
        return os.path.abspath(self._workdir)

    def clear_workdir(self):
        """Clear the workdir"""
        if self._workdir is None:
            return

        workdir = self.get_workdir()

        for f in ['params.pickle', 'data.npy', 'train.npy', 'y_pred.npy', 'ks.pickle', 'out']:
            try:
                os.remove(workdir + '/' + f)
            except FileNotFoundError:
                pass

    def abort_process(self):
        if QtWidgets.QMessageBox.warning(self, 'Abort?',
                                         'Confirm abort',
                                         QtWidgets.QMessageBox.Yes,
                                         QtWidgets.QMessageBox.No) == QtWidgets.QMessageBox.No:
            return False
        self.process.finished.disconnect(self.process_finished)
        self.terminate_qprocess()
        self.control_widget.set_inactive()

        return True

    def terminate_qprocess(self):
        """Terminate running QProcess"""
        try:
            py_proc = psutil.Process(self.process.pid()).children()[0].pid
        except psutil.NoSuchProcess:
            return
        children = psutil.Process(py_proc).children()

        if not IS_WINDOWS:
            os.kill(py_proc, SIGKILL)

            for child in children:
                os.kill(child.pid, SIGKILL)

        elif IS_WINDOWS:
            TerminateProcess(py_proc, -1)
            CloseHandle(py_proc)

            for child in children:
                TerminateProcess(child.pid, -1)
                CloseHandle(child.pid)

    def process_finished(self):
        """Set the plots when the external clustering QProcess finishes"""
        self.control_widget.set_inactive()

        if open(self.params['out'], 'r').read() == 0:
            return

        if self.params['do_single']:
            self.load_output_single()
        elif self.params['do_grid']:
            self.load_output_grid()

    def load_output_single(self):
        ks_path = os.path.join(self.params['workdir'], 'ks.pickle')
        self.ks = pickle.load(open(ks_path, 'rb'))

        train_path = os.path.join(self.params['workdir'], 'train.npy')
        self.train_data = np.load(train_path)

        y_pred_path = os.path.join(self.params['workdir'], 'y_pred.npy')
        self.y_pred = np.load(y_pred_path)

        self.n_clusters = self.params['kwargs']['n_clusters']
        self.cluster_centers = self.ks.cluster_centers_

        self.update_output()

    def load_output_grid(self):
        n_clusters_out_path = os.path.join(self.params['workdir'], 'n_clusters_out.npy')
        self.n_clusters_out = np.load(n_clusters_out_path)

        ksg_path = os.path.join(self.params['workdir'], 'kga.json')
        self._ksgrid_json = json.load(open(ksg_path, 'r'))

        self.train_data = np.empty(0)

    def update_ksgrid_selection(self, ix: Tuple[int, int]):
        self.ks = self.ksgrid.T[ix]

        self.y_pred = self.ks.predict(
            self.pad_input_data(self.input_arrays, method='fill-size')
        )
        self.n_clusters = self.ks.n_clusters
        self.cluster_centers = self.ks.cluster_centers_

        self.update_output()

    def update_output(self):
        padded = self.pad_input_data(self.input_arrays)
        scaled = TimeSeriesScalerMeanVariance().fit_transform(padded)[:, :, 0]
        means = []
        for c_ix in range(self.n_clusters):
            members = scaled[self.y_pred == c_ix]
            means.append(np.mean(members, axis=0))
        self.cluster_means = np.array(means)

        self.set_n_clusters_list()

        self.transmission.df['KSHAPE_CLUSTER'] = self.y_pred

        self.proportions_widget.plot.set_input(self.transmission)

        self.update_plot_means()

        self.send_output_transmission()
        self.finished = True

    def send_output_transmission(self):
        """Send output Transmission containing cluster labels"""
        params = self.params['kwargs']
        params.update({'data_column': self.data_column})

        t = self.transmission.copy()
        t.history_trace.add_operation('all', operation='kshape', parameters=params)

        self.sig_output_changed.emit(t)

    def save_plot(self, path):
        t = self.transmission.copy()

        params = self.params['kwargs']
        params.update({'data_column': self.data_column})

        self.transmission.history_trace.add_operation('all', operation='kshape', parameters=params)

        super(KShapeWidget, self).save_plot(path)

        self.transmission = t

    def set_n_clusters_list(self):
        self.control_widget.ui.listWidgetClusterNumber.clear()
        self.control_widget.ui.listWidgetClusterNumber.addItems(list(map(str, range(self.n_clusters))))
        self.control_widget.ui.listWidgetClusterNumber.setCurrentRow(0)

    def set_plot_opts(self, opts: dict):
        """Set the plot options"""
        prop_opts = opts.pop('proportion_opts')
        if prop_opts is not None:
            self.plot_proportions.set_plot_opts(prop_opts)

        ui_state = opts.pop('ui_state')

        if ui_state is not None:
            ix = self.control_widget.ui.comboBoxDataColumn.findText(ui_state['data_column'])
            self.control_widget.ui.comboBoxDataColumn.setCurrentIndex(ix)
            self.control_widget.ui.checkBoxShowKShapeCenters.setChecked(ui_state['show_centers'])
            self.control_widget.ui.spinBoxMaxNumCurves.setValue(ui_state['max_num_curves'])
            ix = self.control_widget.ui.comboBoxErrorBand.findText(ui_state['error_band'])
            self.control_widget.ui.comboBoxErrorBand.setCurrentIndex(ix)
            self.control_widget.ui.radioButtonXZeroZero.setChecked(ui_state['x-zero_zero'])
            self.control_widget.ui.radioButtonXZeroMaxima.setChecked(ui_state['x-zero_maxima'])

            self.data_column = ui_state['data_column']

        self.input_arrays = self.transmission.df[self.data_column].values

        for k in opts.keys():
            setattr(self, k, opts[k])

        self.set_n_clusters_list()

    def get_plot_opts(self, drop: bool) -> dict:
        """
        Get the plot_raw options

        :param drop: Unused for this Plot Widget, doesn't matter
        """
        opts = {'proportion_opts':  self.plot_proportions.get_plot_opts(drop=True),
                'params':           self.params,
                'n_clusters':       self.n_clusters,
                'train_data':       self.train_data.tolist(),
                'y_pred':           self.y_pred.tolist(),
                'cluster_centers':  self.cluster_centers.tolist(),
                'cluster_means':    self.cluster_means.tolist(),
                '_ksgrid_json':     self._ksgrid_json,
                'ui_state':
                                {'data_column':     self.data_column,
                                 'show_centers':    self.control_widget.ui.checkBoxShowKShapeCenters.isChecked(),
                                 'max_num_curves':  self.control_widget.ui.spinBoxMaxNumCurves.value(),
                                 'error_band':      self.control_widget.ui.comboBoxErrorBand.currentText(),
                                 'x-zero_zero':     self.control_widget.ui.radioButtonXZeroZero.isChecked(),
                                 'x-zero_maxima':   self.control_widget.ui.radioButtonXZeroMaxima.isChecked()
                                 }
                }

        return opts

    @present_exceptions('Plotting error', 'The following error occurred when plotting the raw curves')
    def update_plot(self, *args, show_centers=True, **kwargs):
        """Update the curve plots (raw samples)"""
        max_num_curves = self.control_widget.ui.spinBoxMaxNumCurves.value()
        show_centers = self.control_widget.ui.checkBoxShowKShapeCenters.isChecked()

        if self.control_widget.ui.listWidgetClusterNumber.currentItem() is None:
            self.control_widget.ui.listWidgetClusterNumber.setCurrentRow(0)

        cluster_num = int(self.control_widget.ui.listWidgetClusterNumber.currentItem().text())

        self.plot_raw.ax.cla()

        padded = self.pad_input_data(self.input_arrays)
        scaled = TimeSeriesScalerMeanVariance().fit_transform(padded)[:, :, 0]

        members = scaled[self.y_pred == cluster_num]
        # Plot only 'max_num_curves' to prevent the plot_raw from being overcrowded and hard to visualize
        n_samples = min(members.shape[0], max_num_curves)
        samples = members[np.random.choice(members.shape[0], size=n_samples, replace=False)]

        for sample in samples:
            self.plot_raw.ax.plot(sample.ravel(), 'k-', alpha=0.2)

        if show_centers:
            center = self.cluster_centers[cluster_num].ravel()
            self.plot_raw.ax.plot(center, 'r-')

        low = np.min(samples)
        high = np.max(samples)

        self.plot_raw.ax.set_ylim(low, high)

        self.plot_raw.ax.set_title('Cluster ' + str(cluster_num))

        self.plot_raw.draw()

    @present_exceptions('Plotting error', 'The following error occurred when plotting the means')
    def update_plot_means(self, *args, **kwargs):
        """Update the means plot"""
        padded = self.pad_input_data(self.input_arrays, 'fill-size')
        scaled = TimeSeriesScalerMeanVariance().fit_transform(padded)[:, :, 0]

        if self.control_widget.ui.radioButtonXZeroZero.isChecked():
            xzero = 'zero'
        elif self.control_widget.ui.radioButtonXZeroMaxima.isChecked():
            xzero = 'maxima'
        else:
            raise ValueError('Must select an option for set x = 0 at')

        if self.control_widget.ui.comboBoxErrorBand.currentText() == 'standard deviation':
            ci = 'sd'
        elif self.control_widget.ui.comboBoxErrorBand.currentText() == 'confidence interval':
            ci = 95
        elif self.control_widget.ui.comboBoxErrorBand.currentText() == 'None':
            ci = None

        self.plot_means.set_plots(scaled, self.n_clusters, self.y_pred, xzero_pos=xzero, error_band=ci)
        self.plot_means.show()

    # Need to plot the means as well just once
    def open_plot(self, ptrn_path: str, proj_path: str) -> Union[Tuple[str, str], None]:
        super(KShapeWidget, self).open_plot(ptrn_path, proj_path)
        self.update_plot_means()
        self.input_disconnect()
        return ptrn_path, proj_path

    def input_disconnect(self):
        """Disconnect the input from the flowchart"""
        self.input_connected = False
        self.control_widget.ui.labelConnectedToFlowchart.setText('INPUT DISCONNECTED')
        self.control_widget.ui.pushButtonReconnectFlowchartInput.setVisible(True)

    def input_connect(self):
        """Connect the input to the flowchart"""
        self.input_connected = True
        self.control_widget.ui.labelConnectedToFlowchart.setText('INPUT CONNECTED TO FLOWCHART')
        self.control_widget.ui.pushButtonReconnectFlowchartInput.setVisible(False)

    def closeEvent(self, QCloseEvent):
        if self.abort_process():
            QCloseEvent.accept()
        else:
            QCloseEvent.ignore()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# Copyright (C) 2019 Kushal Kolar
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <https://www.gnu.org/licenses/>.


from fcsugar import *
from ..containers import TransmissionContainer
import numpy as np
from tslearn.clustering import KShape
from typing import *


@node
def kshape(container: TransmissionContainer,
           data_column: str,
           n_clusters: int,
           max_iter: int = 300,
           tol: float = 1e-6,
           n_init: int = 1,
           verbose: bool = True,
           random_state: Union[int, None] = None,
           init: np.ndarray = 'random',
           centroid_seeds: np.ndarray = None):
    """

    :param container:
    :param data_column:
    :param n_clusters:
    :param max_iter:
    :param tol:
    :param n_init:
    :param verbose:
    :param random_state:
    :param init:
    :param centroid_seeds: arrays of shape [n_clusters, ts_size]
    :return:
    """
    if centroid_seeds is not None:
        init = np.swapaxes(np.array([centroid_seeds]).T, 0, 1)

    ks = KShape(n_clusters=n_clusters, max_iter=max_iter,
                tol=tol, n_init=n_init, verbose=verbose,
                random_state=random_state, init=init)

    X = np.vstack(container.df[data_column].values)

    y = ks.fit_predict(X)

    container.df['KSHAPE_CLUSTER'] = y

    return container



from ..Qt import QtGui, QtCore
from ast import literal_eval


class KwargPlainTextEdit(QtGui.QPlainTextEdit):
    """Just a plain text entry widget with helper functions to use for passing kwargs"""
    def __init__(self, parent=None, *args, **kwargs):
        QtGui.QPlainTextEdit.__init__(self, parent, *args, **kwargs)

    def get_kwargs(self) -> dict:
        t = self.toPlainText()

        if t == '':
            return dict()

        return dict((k, literal_eval(v)) for k, v in (pair.split('=') for pair in t))

    def set_kwargs(self, d: dict):
        t = ', '.join("%s=%r" % (k, v) for (k, v) in d.items())
        self.setPlainText(t)



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './beeswarm_plot_controls.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_BeeswarmControls(object):
    def setupUi(self, BeeswarmControls):
        BeeswarmControls.setObjectName("BeeswarmControls")
        BeeswarmControls.resize(279, 169)
        self.horizontalSliderSpotSize = QtWidgets.QSlider(BeeswarmControls)
        self.horizontalSliderSpotSize.setGeometry(QtCore.QRect(10, 60, 231, 20))
        self.horizontalSliderSpotSize.setMinimum(1)
        self.horizontalSliderSpotSize.setProperty("value", 10)
        self.horizontalSliderSpotSize.setOrientation(QtCore.Qt.Horizontal)
        self.horizontalSliderSpotSize.setObjectName("horizontalSliderSpotSize")
        self.btnTraceDatapoint = QtWidgets.QPushButton(BeeswarmControls)
        self.btnTraceDatapoint.setGeometry(QtCore.QRect(10, 90, 120, 36))
        self.btnTraceDatapoint.setObjectName("btnTraceDatapoint")
        self.pushButtonSummaryStats = QtWidgets.QPushButton(BeeswarmControls)
        self.pushButtonSummaryStats.setGeometry(QtCore.QRect(10, 130, 176, 36))
        self.pushButtonSummaryStats.setObjectName("pushButtonSummaryStats")
        self.layoutWidget = QtWidgets.QWidget(BeeswarmControls)
        self.layoutWidget.setGeometry(QtCore.QRect(10, 30, 136, 22))
        self.layoutWidget.setObjectName("layoutWidget")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.layoutWidget)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_3 = QtWidgets.QLabel(self.layoutWidget)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_3.addWidget(self.label_3)
        self.labelSpotSize = QtWidgets.QLabel(self.layoutWidget)
        self.labelSpotSize.setObjectName("labelSpotSize")
        self.horizontalLayout_3.addWidget(self.labelSpotSize)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)

        self.retranslateUi(BeeswarmControls)
        self.horizontalSliderSpotSize.valueChanged['int'].connect(self.labelSpotSize.setNum)
        QtCore.QMetaObject.connectSlotsByName(BeeswarmControls)

    def retranslateUi(self, BeeswarmControls):
        _translate = QtCore.QCoreApplication.translate
        BeeswarmControls.setWindowTitle(_translate("BeeswarmControls", "Form"))
        self.btnTraceDatapoint.setText(_translate("BeeswarmControls", "Trace Datapoint"))
        self.pushButtonSummaryStats.setText(_translate("BeeswarmControls", "View Summary Statistics"))
        self.label_3.setText(_translate("BeeswarmControls", "Spot size: "))
        self.labelSpotSize.setText(_translate("BeeswarmControls", "10"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'system_config.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(675, 531)
        Form.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(Form)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.spinBoxThreads = QtWidgets.QSpinBox(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.spinBoxThreads.sizePolicy().hasHeightForWidth())
        self.spinBoxThreads.setSizePolicy(sizePolicy)
        self.spinBoxThreads.setMinimum(1)
        self.spinBoxThreads.setMaximum(999)
        self.spinBoxThreads.setObjectName("spinBoxThreads")
        self.horizontalLayout.addWidget(self.spinBoxThreads)
        self.horizontalLayout_4.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_3 = QtWidgets.QLabel(Form)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_2.addWidget(self.label_3)
        self.lineEditPythonCall = QtWidgets.QLineEdit(Form)
        self.lineEditPythonCall.setObjectName("lineEditPythonCall")
        self.horizontalLayout_2.addWidget(self.lineEditPythonCall)
        self.horizontalLayout_4.addLayout(self.horizontalLayout_2)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.checkBoxUseCUDA = QtWidgets.QCheckBox(Form)
        self.checkBoxUseCUDA.setObjectName("checkBoxUseCUDA")
        self.horizontalLayout_6.addWidget(self.checkBoxUseCUDA)
        self.pushButtonCUDAError = QtWidgets.QPushButton(Form)
        self.pushButtonCUDAError.setObjectName("pushButtonCUDAError")
        self.horizontalLayout_6.addWidget(self.pushButtonCUDAError)
        self.horizontalLayout_8.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_5 = QtWidgets.QLabel(Form)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_7.addWidget(self.label_5)
        self.lineEditWorkDir = QtWidgets.QLineEdit(Form)
        self.lineEditWorkDir.setObjectName("lineEditWorkDir")
        self.horizontalLayout_7.addWidget(self.lineEditWorkDir)
        self.horizontalLayout_8.addLayout(self.horizontalLayout_7)
        self.verticalLayout.addLayout(self.horizontalLayout_8)
        self.label_2 = QtWidgets.QLabel(Form)
        self.label_2.setObjectName("label_2")
        self.verticalLayout.addWidget(self.label_2)
        self.plainTextEditPrefixCommands = QtWidgets.QPlainTextEdit(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.plainTextEditPrefixCommands.sizePolicy().hasHeightForWidth())
        self.plainTextEditPrefixCommands.setSizePolicy(sizePolicy)
        self.plainTextEditPrefixCommands.setPlainText("")
        self.plainTextEditPrefixCommands.setObjectName("plainTextEditPrefixCommands")
        self.verticalLayout.addWidget(self.plainTextEditPrefixCommands)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_4 = QtWidgets.QLabel(Form)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_5.addWidget(self.label_4)
        self.lineEditCustomModulesDir = QtWidgets.QLineEdit(Form)
        self.lineEditCustomModulesDir.setObjectName("lineEditCustomModulesDir")
        self.horizontalLayout_5.addWidget(self.lineEditCustomModulesDir)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.pushButtonResetDefaults = QtWidgets.QPushButton(Form)
        self.pushButtonResetDefaults.setObjectName("pushButtonResetDefaults")
        self.horizontalLayout_3.addWidget(self.pushButtonResetDefaults)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem)
        self.btnReloadConfigFile = QtWidgets.QPushButton(Form)
        self.btnReloadConfigFile.setObjectName("btnReloadConfigFile")
        self.horizontalLayout_3.addWidget(self.btnReloadConfigFile)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.btnClose = QtWidgets.QPushButton(Form)
        self.btnClose.setObjectName("btnClose")
        self.horizontalLayout_3.addWidget(self.btnClose)
        self.btnApply = QtWidgets.QPushButton(Form)
        self.btnApply.setObjectName("btnApply")
        self.horizontalLayout_3.addWidget(self.btnApply)
        self.verticalLayout.addLayout(self.horizontalLayout_3)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)
        Form.setTabOrder(self.spinBoxThreads, self.lineEditPythonCall)
        Form.setTabOrder(self.lineEditPythonCall, self.checkBoxUseCUDA)
        Form.setTabOrder(self.checkBoxUseCUDA, self.pushButtonCUDAError)
        Form.setTabOrder(self.pushButtonCUDAError, self.lineEditWorkDir)
        Form.setTabOrder(self.lineEditWorkDir, self.plainTextEditPrefixCommands)
        Form.setTabOrder(self.plainTextEditPrefixCommands, self.lineEditCustomModulesDir)
        Form.setTabOrder(self.lineEditCustomModulesDir, self.pushButtonResetDefaults)
        Form.setTabOrder(self.pushButtonResetDefaults, self.btnReloadConfigFile)
        Form.setTabOrder(self.btnReloadConfigFile, self.btnClose)
        Form.setTabOrder(self.btnClose, self.btnApply)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label.setText(_translate("Form", "Maximum number of threads to use:"))
        self.label_3.setText(_translate("Form", "Python call:"))
        self.lineEditPythonCall.setToolTip(_translate("Form", "Depending on how you have installed Mesmerize \n"
"you may wish to call python or python3.\n"
"Python3 should work in all circumstances.\n"
"In certain situations such as when you have\n"
" installed Mesmerize from a snap there is no \n"
"symbolic link to python3."))
        self.lineEditPythonCall.setText(_translate("Form", "python3"))
        self.checkBoxUseCUDA.setText(_translate("Form", "Use CUDA when possible"))
        self.pushButtonCUDAError.setText(_translate("Form", "See details"))
        self.label_5.setText(_translate("Form", "Work dir"))
        self.label_2.setText(_translate("Form", "Any prior commands to run for computations performed in separate processes (such as batch items)"))
        self.label_4.setText(_translate("Form", "Plugins directory (Optional):"))
        self.pushButtonResetDefaults.setText(_translate("Form", "Reset defaults"))
        self.btnReloadConfigFile.setText(_translate("Form", "Reload Config File"))
        self.btnClose.setText(_translate("Form", "Close"))
        self.btnApply.setText(_translate("Form", "Apply"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './control_widget.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Controls(object):
    def setupUi(self, Controls):
        Controls.setObjectName("Controls")
        Controls.resize(325, 851)
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout(self.dockWidgetContents)
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout()
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.dockWidgetContents)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.combo_categorical_column = ComboBox(self.dockWidgetContents)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.combo_categorical_column.sizePolicy().hasHeightForWidth())
        self.combo_categorical_column.setSizePolicy(sizePolicy)
        self.combo_categorical_column.setObjectName("combo_categorical_column")
        self.horizontalLayout.addWidget(self.combo_categorical_column)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.label_2 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_2.setObjectName("label_2")
        self.verticalLayout_2.addWidget(self.label_2)
        self.cmap_img = ColormapListWidget(self.dockWidgetContents)
        self.cmap_img.setMaximumSize(QtCore.QSize(16777215, 150))
        self.cmap_img.setObjectName("cmap_img")
        self.verticalLayout_2.addWidget(self.cmap_img)
        self.label_3 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_3.setObjectName("label_3")
        self.verticalLayout_2.addWidget(self.label_3)
        self.cmap_patches = ColormapListWidget(self.dockWidgetContents)
        self.cmap_patches.setMaximumSize(QtCore.QSize(16777215, 150))
        self.cmap_patches.setObjectName("cmap_patches")
        self.verticalLayout_2.addWidget(self.cmap_patches)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_5 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_2.addWidget(self.label_5)
        self.radioButtonMax = QtWidgets.QRadioButton(self.dockWidgetContents)
        self.radioButtonMax.setObjectName("radioButtonMax")
        self.horizontalLayout_2.addWidget(self.radioButtonMax)
        self.radioButtonStd = QtWidgets.QRadioButton(self.dockWidgetContents)
        self.radioButtonStd.setChecked(True)
        self.radioButtonStd.setObjectName("radioButtonStd")
        self.horizontalLayout_2.addWidget(self.radioButtonStd)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.checkBoxFill = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBoxFill.setObjectName("checkBoxFill")
        self.verticalLayout_2.addWidget(self.checkBoxFill)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_6 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_3.addWidget(self.label_6)
        self.doubleSpinBoxLineWidth = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBoxLineWidth.setMaximum(50.0)
        self.doubleSpinBoxLineWidth.setSingleStep(0.25)
        self.doubleSpinBoxLineWidth.setProperty("value", 0.5)
        self.doubleSpinBoxLineWidth.setObjectName("doubleSpinBoxLineWidth")
        self.horizontalLayout_3.addWidget(self.doubleSpinBoxLineWidth)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_3.addItem(spacerItem1)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_7 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_4.addWidget(self.label_7)
        self.doubleSpinBoxAlpha = QtWidgets.QDoubleSpinBox(self.dockWidgetContents)
        self.doubleSpinBoxAlpha.setMaximum(1.0)
        self.doubleSpinBoxAlpha.setSingleStep(0.05)
        self.doubleSpinBoxAlpha.setProperty("value", 0.7)
        self.doubleSpinBoxAlpha.setObjectName("doubleSpinBoxAlpha")
        self.horizontalLayout_4.addWidget(self.doubleSpinBoxAlpha)
        spacerItem2 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_4.addItem(spacerItem2)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.label_4 = QtWidgets.QLabel(self.dockWidgetContents)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_2.addWidget(self.label_4)
        self.list_widget_samples = ListWidget(self.dockWidgetContents)
        self.list_widget_samples.setObjectName("list_widget_samples")
        self.verticalLayout_2.addWidget(self.list_widget_samples)
        self.checkBoxLiveUpdate = QtWidgets.QCheckBox(self.dockWidgetContents)
        self.checkBoxLiveUpdate.setObjectName("checkBoxLiveUpdate")
        self.verticalLayout_2.addWidget(self.checkBoxLiveUpdate)
        self.pushButtonUpdatePlot = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonUpdatePlot.setObjectName("pushButtonUpdatePlot")
        self.verticalLayout_2.addWidget(self.pushButtonUpdatePlot)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.pushButtonSave = QtWidgets.QPushButton(self.dockWidgetContents)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.pushButtonSave.setFont(font)
        self.pushButtonSave.setObjectName("pushButtonSave")
        self.horizontalLayout_5.addWidget(self.pushButtonSave)
        self.pushButtonLoad = QtWidgets.QPushButton(self.dockWidgetContents)
        self.pushButtonLoad.setObjectName("pushButtonLoad")
        self.horizontalLayout_5.addWidget(self.pushButtonLoad)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6.addLayout(self.verticalLayout_2)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.groupBox = QtWidgets.QGroupBox(self.dockWidgetContents)
        self.groupBox.setObjectName("groupBox")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.groupBox)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.spinBoxZLevel = QtWidgets.QSpinBox(self.groupBox)
        self.spinBoxZLevel.setObjectName("spinBoxZLevel")
        self.verticalLayout_3.addWidget(self.spinBoxZLevel)
        self.verticalSliderZLevel = QtWidgets.QSlider(self.groupBox)
        self.verticalSliderZLevel.setPageStep(5)
        self.verticalSliderZLevel.setOrientation(QtCore.Qt.Vertical)
        self.verticalSliderZLevel.setInvertedAppearance(True)
        self.verticalSliderZLevel.setObjectName("verticalSliderZLevel")
        self.verticalLayout_3.addWidget(self.verticalSliderZLevel)
        self.verticalLayout.addWidget(self.groupBox)
        self.horizontalLayout_6.addLayout(self.verticalLayout)
        Controls.setWidget(self.dockWidgetContents)

        self.retranslateUi(Controls)
        self.verticalSliderZLevel.valueChanged['int'].connect(self.spinBoxZLevel.setValue)
        self.spinBoxZLevel.valueChanged['int'].connect(self.verticalSliderZLevel.setValue)
        QtCore.QMetaObject.connectSlotsByName(Controls)

    def retranslateUi(self, Controls):
        _translate = QtCore.QCoreApplication.translate
        Controls.setWindowTitle(_translate("Controls", "Co&ntrols"))
        self.label.setText(_translate("Controls", "Patch Labels"))
        self.label_2.setText(_translate("Controls", "Image Colormap"))
        self.label_3.setText(_translate("Controls", "Patches Colormap"))
        self.label_5.setText(_translate("Controls", "Projection"))
        self.radioButtonMax.setText(_translate("Controls", "&Max"))
        self.radioButtonStd.setText(_translate("Controls", "Std"))
        self.checkBoxFill.setText(_translate("Controls", "Fill Patches"))
        self.label_6.setText(_translate("Controls", "Line width"))
        self.label_7.setText(_translate("Controls", "Alpha"))
        self.label_4.setText(_translate("Controls", "Samples"))
        self.checkBoxLiveUpdate.setText(_translate("Controls", "Live update from flowchart"))
        self.pushButtonUpdatePlot.setText(_translate("Controls", "Update Plot"))
        self.pushButtonSave.setText(_translate("Controls", "Save"))
        self.pushButtonLoad.setText(_translate("Controls", "Load"))
        self.groupBox.setTitle(_translate("Controls", "z-level"))

from ....pyqtgraphCore.widgets.ComboBox import ComboBox
from ....pyqtgraphCore.widgets.ListWidget import ListWidget
from ...utils import ColormapListWidget



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007

A back-end module for managing most project related functions, such as the root dataframe and all children,
adding rows to the root dataframe, updating child dataframes, and updating the project configuration.
"""

from shutil import copy2
from glob import glob
from os import makedirs


def main(proj_dir: str, dest_dir: str):
    print(proj_dir, dest_dir)



import numpy as np
from typing import *
from caiman.base.rois import register_multisession
from ..common import get_project_manager
import pickle
import os
import tifffile


def assign_component_ids(assignments: np.ndarray) -> List[Dict[int, int]]:#, idx_components_sessions: List[np.ndarray]):
    """
    Returns a list of dicts to map component indices from a session to a cell ID.
    list index is session number.
    :param assignments:
    """
    mappings = [dict()] * assignments.shape[1]
    for cell_id in assignments.shape[0]:
        for session in assignments.shape[1]:
            idx_component = assignments[cell_id, session]
            mappings[session][idx_component] = cell_id

    return mappings


def register_animal(animal_id: str, session_column: str):
    df = get_project_manager().dataframe
    animal_df = df[df.SampleID.apply(lambda sid: sid.split('-_-')[0] == animal_id)]
    animal_df = animal_df.copy()

    animal_df.drop_duplicates(subset=session_column, keep='first', inplace=True)

    animal_df.sort_values(by=[session_column], inplace=True).reset_index(drop=True)

    data_paths = [os.path.join(get_project_manager().root_dir, path) for path in animal_df.ImgInfoPath.tolist()]


def register_sessions(data_paths: List[str]) -> np.ndarray:
    """

    :param data_paths:
    :return: assignments array
    """
    As = []
    templates = []

    for path in data_paths:
        p = pickle.load(open(path, 'rb'))
        As.append(p['roi_states']['cnmf_output']['cnmA'])





# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'batch_manager_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(999, 759)
        self.verticalLayout_6 = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout_6.setObjectName("verticalLayout_6")
        self.labelBatchPath = QtWidgets.QLabel(Form)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.labelBatchPath.sizePolicy().hasHeightForWidth())
        self.labelBatchPath.setSizePolicy(sizePolicy)
        self.labelBatchPath.setMinimumSize(QtCore.QSize(0, 30))
        self.labelBatchPath.setMaximumSize(QtCore.QSize(16777215, 40))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.labelBatchPath.setFont(font)
        self.labelBatchPath.setText("")
        self.labelBatchPath.setTextInteractionFlags(QtCore.Qt.LinksAccessibleByMouse|QtCore.Qt.TextSelectableByKeyboard|QtCore.Qt.TextSelectableByMouse)
        self.labelBatchPath.setObjectName("labelBatchPath")
        self.verticalLayout_6.addWidget(self.labelBatchPath)
        self.splitter_3 = QtWidgets.QSplitter(Form)
        self.splitter_3.setOrientation(QtCore.Qt.Horizontal)
        self.splitter_3.setObjectName("splitter_3")
        self.splitter = QtWidgets.QSplitter(self.splitter_3)
        self.splitter.setOrientation(QtCore.Qt.Vertical)
        self.splitter.setObjectName("splitter")
        self.layoutWidget = QtWidgets.QWidget(self.splitter)
        self.layoutWidget.setObjectName("layoutWidget")
        self.verticalLayout_5 = QtWidgets.QVBoxLayout(self.layoutWidget)
        self.verticalLayout_5.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_5.setObjectName("verticalLayout_5")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.listWidgetItemNumbers = QtWidgets.QListWidget(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(1)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listWidgetItemNumbers.sizePolicy().hasHeightForWidth())
        self.listWidgetItemNumbers.setSizePolicy(sizePolicy)
        self.listWidgetItemNumbers.setMinimumSize(QtCore.QSize(30, 0))
        self.listWidgetItemNumbers.setObjectName("listWidgetItemNumbers")
        self.horizontalLayout.addWidget(self.listWidgetItemNumbers)
        self.listwBatch = QtWidgets.QListWidget(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(10)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.listwBatch.sizePolicy().hasHeightForWidth())
        self.listwBatch.setSizePolicy(sizePolicy)
        self.listwBatch.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.listwBatch.setObjectName("listwBatch")
        self.horizontalLayout.addWidget(self.listwBatch)
        self.verticalLayout_5.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(self.layoutWidget)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.lineEditFindItem = QtWidgets.QLineEdit(self.layoutWidget)
        self.lineEditFindItem.setObjectName("lineEditFindItem")
        self.horizontalLayout_2.addWidget(self.lineEditFindItem)
        self.verticalLayout_5.addLayout(self.horizontalLayout_2)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.btnStart = QtWidgets.QPushButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnStart.sizePolicy().hasHeightForWidth())
        self.btnStart.setSizePolicy(sizePolicy)
        self.btnStart.setMinimumSize(QtCore.QSize(55, 0))
        self.btnStart.setMaximumSize(QtCore.QSize(16777215, 50))
        self.btnStart.setBaseSize(QtCore.QSize(60, 0))
        self.btnStart.setObjectName("btnStart")
        self.horizontalLayout_4.addWidget(self.btnStart)
        self.btnStartAtSelection = QtWidgets.QPushButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnStartAtSelection.sizePolicy().hasHeightForWidth())
        self.btnStartAtSelection.setSizePolicy(sizePolicy)
        self.btnStartAtSelection.setMinimumSize(QtCore.QSize(125, 0))
        self.btnStartAtSelection.setMaximumSize(QtCore.QSize(16777215, 50))
        self.btnStartAtSelection.setBaseSize(QtCore.QSize(60, 0))
        self.btnStartAtSelection.setObjectName("btnStartAtSelection")
        self.horizontalLayout_4.addWidget(self.btnStartAtSelection)
        self.btnDelete = QtWidgets.QPushButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnDelete.sizePolicy().hasHeightForWidth())
        self.btnDelete.setSizePolicy(sizePolicy)
        self.btnDelete.setMinimumSize(QtCore.QSize(110, 0))
        self.btnDelete.setMaximumSize(QtCore.QSize(16777215, 50))
        self.btnDelete.setObjectName("btnDelete")
        self.horizontalLayout_4.addWidget(self.btnDelete)
        self.btnExportShScripts = QtWidgets.QPushButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnExportShScripts.sizePolicy().hasHeightForWidth())
        self.btnExportShScripts.setSizePolicy(sizePolicy)
        self.btnExportShScripts.setMaximumSize(QtCore.QSize(16777215, 50))
        self.btnExportShScripts.setObjectName("btnExportShScripts")
        self.horizontalLayout_4.addWidget(self.btnExportShScripts)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.btnAbort = QtWidgets.QPushButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnAbort.sizePolicy().hasHeightForWidth())
        self.btnAbort.setSizePolicy(sizePolicy)
        self.btnAbort.setMinimumSize(QtCore.QSize(55, 0))
        self.btnAbort.setMaximumSize(QtCore.QSize(16777215, 50))
        self.btnAbort.setBaseSize(QtCore.QSize(60, 0))
        self.btnAbort.setObjectName("btnAbort")
        self.horizontalLayout_3.addWidget(self.btnAbort)
        self.btnAbort_batch = QtWidgets.QPushButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnAbort_batch.sizePolicy().hasHeightForWidth())
        self.btnAbort_batch.setSizePolicy(sizePolicy)
        self.btnAbort_batch.setMinimumSize(QtCore.QSize(55, 0))
        self.btnAbort_batch.setMaximumSize(QtCore.QSize(16777215, 50))
        self.btnAbort_batch.setBaseSize(QtCore.QSize(60, 0))
        self.btnAbort_batch.setObjectName("btnAbort_batch")
        self.horizontalLayout_3.addWidget(self.btnAbort_batch)
        self.btnNew = QtWidgets.QPushButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnNew.sizePolicy().hasHeightForWidth())
        self.btnNew.setSizePolicy(sizePolicy)
        self.btnNew.setMinimumSize(QtCore.QSize(80, 0))
        self.btnNew.setMaximumSize(QtCore.QSize(16777215, 50))
        self.btnNew.setBaseSize(QtCore.QSize(60, 0))
        self.btnNew.setObjectName("btnNew")
        self.horizontalLayout_3.addWidget(self.btnNew)
        self.btnOpen = QtWidgets.QPushButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnOpen.sizePolicy().hasHeightForWidth())
        self.btnOpen.setSizePolicy(sizePolicy)
        self.btnOpen.setMinimumSize(QtCore.QSize(80, 0))
        self.btnOpen.setMaximumSize(QtCore.QSize(16777215, 50))
        self.btnOpen.setBaseSize(QtCore.QSize(60, 0))
        self.btnOpen.setObjectName("btnOpen")
        self.horizontalLayout_3.addWidget(self.btnOpen)
        self.btnViewInput = QtWidgets.QPushButton(self.layoutWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.MinimumExpanding, QtWidgets.QSizePolicy.MinimumExpanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.btnViewInput.sizePolicy().hasHeightForWidth())
        self.btnViewInput.setSizePolicy(sizePolicy)
        self.btnViewInput.setMinimumSize(QtCore.QSize(75, 0))
        self.btnViewInput.setMaximumSize(QtCore.QSize(16777215, 50))
        self.btnViewInput.setBaseSize(QtCore.QSize(70, 0))
        self.btnViewInput.setObjectName("btnViewInput")
        self.horizontalLayout_3.addWidget(self.btnViewInput)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.verticalLayout_5.addLayout(self.verticalLayout)
        self.checkBoxUseWorkDir = QtWidgets.QCheckBox(self.layoutWidget)
        font = QtGui.QFont()
        font.setPointSize(16)
        font.setBold(True)
        font.setWeight(75)
        self.checkBoxUseWorkDir.setFont(font)
        self.checkBoxUseWorkDir.setObjectName("checkBoxUseWorkDir")
        self.verticalLayout_5.addWidget(self.checkBoxUseWorkDir)
        self.progressBar = QtWidgets.QProgressBar(self.layoutWidget)
        self.progressBar.setMinimum(0)
        self.progressBar.setProperty("value", 0)
        self.progressBar.setTextVisible(True)
        self.progressBar.setTextDirection(QtWidgets.QProgressBar.TopToBottom)
        self.progressBar.setObjectName("progressBar")
        self.verticalLayout_5.addWidget(self.progressBar)
        self.scrollArea = QtWidgets.QScrollArea(self.splitter)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.scrollArea.sizePolicy().hasHeightForWidth())
        self.scrollArea.setSizePolicy(sizePolicy)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 644, 302))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.textBrowserItemInfo = QtWidgets.QTextBrowser(self.scrollAreaWidgetContents)
        self.textBrowserItemInfo.setObjectName("textBrowserItemInfo")
        self.verticalLayout_2.addWidget(self.textBrowserItemInfo)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.splitter_2 = QtWidgets.QSplitter(self.splitter_3)
        self.splitter_2.setOrientation(QtCore.Qt.Vertical)
        self.splitter_2.setObjectName("splitter_2")
        self.scrollAreaStdOut = QtWidgets.QScrollArea(self.splitter_2)
        self.scrollAreaStdOut.setEnabled(True)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.scrollAreaStdOut.sizePolicy().hasHeightForWidth())
        self.scrollAreaStdOut.setSizePolicy(sizePolicy)
        self.scrollAreaStdOut.setWidgetResizable(True)
        self.scrollAreaStdOut.setObjectName("scrollAreaStdOut")
        self.scrollAreaWidgetContents_4 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_4.setGeometry(QtCore.QRect(0, 0, 334, 346))
        self.scrollAreaWidgetContents_4.setObjectName("scrollAreaWidgetContents_4")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents_4)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.textBrowserStdOut = QtWidgets.QTextBrowser(self.scrollAreaWidgetContents_4)
        self.textBrowserStdOut.setObjectName("textBrowserStdOut")
        self.verticalLayout_4.addWidget(self.textBrowserStdOut)
        self.scrollAreaStdOut.setWidget(self.scrollAreaWidgetContents_4)
        self.scrollAreaOutputInfo = QtWidgets.QScrollArea(self.splitter_2)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(2)
        sizePolicy.setVerticalStretch(1)
        sizePolicy.setHeightForWidth(self.scrollAreaOutputInfo.sizePolicy().hasHeightForWidth())
        self.scrollAreaOutputInfo.setSizePolicy(sizePolicy)
        self.scrollAreaOutputInfo.setWidgetResizable(True)
        self.scrollAreaOutputInfo.setObjectName("scrollAreaOutputInfo")
        self.scrollAreaWidgetContents_6 = QtWidgets.QWidget()
        self.scrollAreaWidgetContents_6.setGeometry(QtCore.QRect(0, 0, 334, 346))
        self.scrollAreaWidgetContents_6.setObjectName("scrollAreaWidgetContents_6")
        self.verticalLayout_3 = QtWidgets.QVBoxLayout(self.scrollAreaWidgetContents_6)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.textBrowserOutputInfo = QtWidgets.QTextBrowser(self.scrollAreaWidgetContents_6)
        self.textBrowserOutputInfo.setObjectName("textBrowserOutputInfo")
        self.verticalLayout_3.addWidget(self.textBrowserOutputInfo)
        self.scrollAreaOutputInfo.setWidget(self.scrollAreaWidgetContents_6)
        self.verticalLayout_6.addWidget(self.splitter_3)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)
        Form.setTabOrder(self.listWidgetItemNumbers, self.listwBatch)
        Form.setTabOrder(self.listwBatch, self.lineEditFindItem)
        Form.setTabOrder(self.lineEditFindItem, self.btnStart)
        Form.setTabOrder(self.btnStart, self.btnStartAtSelection)
        Form.setTabOrder(self.btnStartAtSelection, self.btnDelete)
        Form.setTabOrder(self.btnDelete, self.btnExportShScripts)
        Form.setTabOrder(self.btnExportShScripts, self.btnAbort)
        Form.setTabOrder(self.btnAbort, self.btnAbort_batch)
        Form.setTabOrder(self.btnAbort_batch, self.btnNew)
        Form.setTabOrder(self.btnNew, self.btnOpen)
        Form.setTabOrder(self.btnOpen, self.btnViewInput)
        Form.setTabOrder(self.btnViewInput, self.checkBoxUseWorkDir)
        Form.setTabOrder(self.checkBoxUseWorkDir, self.scrollArea)
        Form.setTabOrder(self.scrollArea, self.textBrowserItemInfo)
        Form.setTabOrder(self.textBrowserItemInfo, self.scrollAreaStdOut)
        Form.setTabOrder(self.scrollAreaStdOut, self.textBrowserStdOut)
        Form.setTabOrder(self.textBrowserStdOut, self.scrollAreaOutputInfo)
        Form.setTabOrder(self.scrollAreaOutputInfo, self.textBrowserOutputInfo)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label.setText(_translate("Form", "Find:"))
        self.btnStart.setText(_translate("Form", "Start"))
        self.btnStartAtSelection.setText(_translate("Form", "Start at selection"))
        self.btnDelete.setText(_translate("Form", "Delete selection"))
        self.btnExportShScripts.setText(_translate("Form", "Export shell scripts"))
        self.btnAbort.setText(_translate("Form", "Abort current item"))
        self.btnAbort_batch.setText(_translate("Form", "Abort batch"))
        self.btnNew.setText(_translate("Form", "New Batch"))
        self.btnOpen.setText(_translate("Form", "Open Batch"))
        self.btnViewInput.setText(_translate("Form", "View Input"))
        self.checkBoxUseWorkDir.setToolTip(_translate("Form", "Use a temporary working directory to process each batch item. Moves output back to batch directory when done. Useful if you have a very fast filesystem with limited capacity."))
        self.checkBoxUseWorkDir.setText(_translate("Form", "Use work dir"))
        self.textBrowserItemInfo.setHtml(_translate("Form", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Noto Sans\'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px; font-family:\'Sans Serif\'; font-size:9pt;\"><br /></p></body></html>"))




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on June 15 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from .datapoint_tracer_pytemplate import *
from ...analysis.history_widget import HistoryTreeWidget
from ...pyqtgraphCore import ImageView, LinearRegionItem, mkColor, PlotDataItem
from uuid import UUID
import pandas as pd
import tifffile
import numpy as np
import pickle
from ...viewer.modules.roi_manager_modules.roi_types import CNMFROI, ManualROI, ScatterROI, VolCNMF, VolMultiCNMFROI
# from ...viewer.core import ViewerWorkEnv, ViewerUtils
from ...common import get_window_manager, get_project_manager
# from common import configuration
import os
from typing import Union, Optional
from ...common.utils import draw_graph
from ...analysis.data_types import HistoryTrace
from copy import deepcopy

region_data_types = ['_pf_uuid', '_ST_uuid']


class DatapointTracerWidget(QtWidgets.QWidget):
    def __init__(self):
        QtWidgets.QWidget.__init__(self)
        self.setWindowTitle('Datapoint Tracer')

        self.uuid = None
        self.row = None
        self.proj_path = None
        self.sample_id = None
        self.previous_sample_id_projection = None
        self.history_trace = None
        self.peak_ix = None
        self.tstart = None
        self.tend = None

        self.is_3d = False

        self.ui = Ui_DatapointTracer()
        self.ui.setupUi(self)
        self.history_widget = HistoryTreeWidget(parent=self.ui.groupBoxInfo)
        self.ui.groupBoxInfo.layout().addWidget(self.history_widget)

        self.pandas_series_widget = HistoryTreeWidget(parent=self.ui.groupBoxInfo)

        self.ui.groupBoxInfo.layout().addWidget(self.pandas_series_widget)
        self.image_view = ImageView()
        self.image_view.tVals = np.arange(0, 100)
        self.image_item = self.image_view.getImageItem()
        self.view = self.ui.graphicsViewImage.addViewBox()
        self.view.setAspectLocked(True)
        self.view.addItem(self.image_item)

        self.peak_region = TimelineLinearRegion(self.ui.graphicsViewPlot)
        self.roi = None

        self.plot_data: np.ndarray = None
        self.plot_data_item: PlotDataItem = None

        self.ui.radioButtonMaxProjection.clicked.connect(lambda x: self.set_image('max'))
        self.ui.radioButtonSTDProjection.clicked.connect(lambda x: self.set_image('std'))

        self.ui.pushButtonOpenInViewer.clicked.connect(self.open_in_viewer)
        self.ui.pushButtonOpenAnalysisGraph.clicked.connect(self.open_analysis_graph)

    def set_widget(self, datapoint_uuid: UUID, data_column_curve: str, row: pd.Series, proj_path: str,
                   history_trace: Optional[list] = None, peak_ix: Optional[int] = None, tstart: Optional[int] = None,
                   tend: Optional[int] = None, roi_color: Optional[Union[str, float, int, tuple]] = 'ff0000',
                   clear_linear_regions: bool = True):
        """
        Set the widget from the datapoint.

        :param datapoint_uuid:      appropriate UUID for the datapoint (such as uuid_curve or _pfeature_uuid)
        :param data_column_curve:   data column containing an array to plot
        :param row:                 DataFrame row that corresponds to the datapoint
        :param proj_path:           root dir of the project the datapoint comes from, used for finding max & std projections
        :param history_trace:       history trace of the datablock the datapoint comes from
        :param peak_ix:             Deprecated
        :param tstart:              lower bounds for drawing `LinearRegionItem`
        :param tend:                upper bounds for drawing `LinearRegionItem`
        :param roi_color:           color for drawing the spatial bounds of the ROI
        """
        self.uuid = datapoint_uuid
        self.row = row
        self.proj_path = proj_path
        self.sample_id = self.row['SampleID']

        self.ui.label_zlevel.clear()

        if isinstance(self.sample_id, pd.Series):
            self.sample_id = self.sample_id.item()

        if not isinstance(self.sample_id, str):
            raise ValueError('SampleID datatype is not str or pandas.Series. '
                             'Something is wrong, it is this datatype :' + str(type(self.sample_id)))

        if history_trace is None:
            self.history_trace = []

        img_info_path = row['ImgInfoPath']
        if isinstance(img_info_path, pd.Series):
            img_info_path = img_info_path.item()
        img_info_path = os.path.join(self.proj_path, img_info_path)
        preprocess_history = pickle.load(open(img_info_path, 'rb'))['history_trace']

        self.history_trace = preprocess_history + history_trace

        self.ui.lineEditUUID.setText(str(self.uuid))
        self.history_widget.fill_widget(self.history_trace)

        row_dict = row.to_dict()
        for k in row_dict.keys():
            row_dict[k] = row_dict[k][row.index.item()]
        self.pandas_series_widget.fill_widget(row_dict)
        # self.pandas_series_widget.collapseAll()

        if self.ui.radioButtonMaxProjection.isChecked():
            self.img_proj = 'max'
        elif self.ui.radioButtonSTDProjection.isChecked():
            self.img_proj = 'std'

        if clear_linear_regions:
            self.peak_region.clear_all()

        if (tstart is not None) and (tend is not None):
            self.peak_region.add_linear_region(tstart, tend, color=mkColor('#a80035'))

        # get the plot data
        try:
            self.plot_data = self.row[data_column_curve].item()
        except:
            self.plot_data = self.row[data_column_curve]

        # get a new pyqtgraph plot data item
        if self.plot_data_item is None:
            self.plot_data_item = self.ui.graphicsViewPlot.plot(self.plot_data)

        # or set the existing one
        else:
            self.plot_data_item.clear()
            self.plot_data_item.setData(self.plot_data)

        self.plot_data_item.setPen('w', width=2)

        self.plot_data_item.setZValue(1)

        if self.roi is not None:
            self.roi.remove_from_viewer()
        try:
            roi_state = self.row['ROI_State'].item()
        except:
            roi_state = self.row['ROI_State']

        if roi_state['roi_type'] in ['CNMFROI', 'ScatterROI']:
            self.set_image(self.img_proj)
            ROIClass = globals()[roi_state['roi_type']]
            self.roi = ROIClass.from_state(curve_plot_item=None, view_box=self.view, state=roi_state)
            self.roi.get_roi_graphics_object().setBrush(mkColor(roi_color))

        elif roi_state['roi_type'] == 'ManualROI':
            self.set_image(self.img_proj)
            self.roi = ManualROI.from_state(curve_plot_item=None, view_box=self.view, state=roi_state)

        elif roi_state['roi_type'] in ['VolCNMF']:
            self.is_3d = True
            self.zcenter = roi_state['zcenter']
            self.set_image(self.img_proj)

            self.roi = VolCNMF.from_state(curve_plot_item=None, view_box=self.view, state=roi_state, zlevel=self.zcenter)
            self.roi.get_roi_graphics_object().setBrush(mkColor(roi_color))

        elif roi_state['roi_type'] in ['VolMultiCNMFROI']:
            self.is_3d = True
            self.zcenter = roi_state['zcenter']
            self.set_image(self.img_proj)

            self.roi = VolMultiCNMFROI.from_state(
                curve_plot_item=None, view_box=self.view, state=roi_state, zlevel=self.zcenter
            )

        self.roi.get_roi_graphics_object().setPen(mkColor(roi_color))
        self.roi.add_to_viewer()

        if peak_ix is not None:
            pass

    def set_image(self, projection: str):
        """
        Set either the max or std projection image

        :param projection: one of either 'max' or 'std'
        """
        if f'{self.sample_id}{projection}' == self.previous_sample_id_projection:
            if not self.is_3d:
                return

        img_uuid = self.row['ImgUUID']

        if isinstance(img_uuid, pd.Series):
            img_uuid = img_uuid.item()

        if not isinstance(img_uuid, str):
            raise ValueError('Datatype for Projection Path must be pandas.Series or str, it is currently : ' + str(
                type(img_uuid)))

        if projection == 'max':
            suffix = '_max_proj'
        elif projection == 'std':
            suffix = '_std_proj'
        else:
            raise ValueError('Can only accept "max" and "std" arguments')

        self.ui.label_zlevel.clear()

        if self.is_3d:
            suffix += f'-{self.zcenter}'
            self.ui.label_zlevel.setText(f'Showing plane #: {self.zcenter}   ')

        img_path = os.path.join(self.proj_path, 'images', f'{self.sample_id}-_-{img_uuid}{suffix}.tiff')

        img = tifffile.imread(img_path)

        # z = np.zeros(img.shape)

        # img = np.dstack((img, z))

        # if img.shape[0] > img.shape[1]:
        #     x, y = (0, 1)
        # else:
        #     x,y = (1, 0)
        vmin = np.nanmin(img)
        vmax = np.nanmedian(img) + (10 * np.nanstd(img))
        self.image_view.setImage(img, axes={'x': 0, 'y': 1}, levels=(vmin, vmax))

        self.previous_sample_id_projection = f'{self.sample_id}{projection}'
        # self.image_item.setImage(img.T.astype(np.uint16))
        # self.image_item.resetTransform()

    def open_in_viewer(self):
        """
        Open the parent Sample of the current datapoint.
        """
        w = get_window_manager().get_new_viewer_window()
        w.open_from_dataframe(proj_path=self.proj_path, row=self.row)

    def open_analysis_graph(self):
        cleaned = HistoryTrace.clean_history_trace(deepcopy(self.history_trace))
        draw_graph(cleaned, view=True)


class TimelineLinearRegion:
    def __init__(self, plot_widget: PlotWidget):
        self.plot_widget = plot_widget
        self.linear_regions = []

    def add_linear_region(self, frame_start: int, frame_end: int, color: QtGui.QColor) -> LinearRegionItem:
        linear_region = LinearRegionItem(values=[frame_start, frame_end],
                                         brush=color, movable=False, bounds=[frame_start, frame_end])
        linear_region.setZValue(0)
        linear_region.lines[0].setPen(mkColor('#000000'))
        linear_region.lines[1].setPen(mkColor('#000000'))

        self.linear_regions.append(linear_region)
        self.plot_widget.addItem(linear_region)

        return linear_region

    def del_linear_region(self, linear_region: LinearRegionItem):
        self.plot_widget.removeItem(linear_region)
        linear_region.deleteLater()
        self.linear_regions.remove(linear_region)

    def clear_all(self):
        for region in self.linear_regions:
            self.plot_widget.removeItem(region)
            region.deleteLater()
        self.linear_regions = []



from .common import *
from sklearn import neighbors
from ....common.configuration import sys_cfg


class KneighborsGraph(CtrlNode):
    """Computes the (weighted) graph of k-Neighbors for points in data\n
    Output column -> _KNGRAPH_<data_column>"""
    nodeName = 'KneighborsGraph'
    uiTemplate = [('data_column', 'combo', {}),

                  ('n_neighbors', 'intSpin',
                   {'max': 1000, 'min': 1, 'value': 2, 'step': 5,
                    'toolTip': 'Number of neighbors for each sample'}),

                  ('mode', 'combo', {'items': ['connectivity', 'distance'],
                                     'toolTip': '‘connectivity’ will return the connectivity matrix with ones and zeros'
                                                '\n‘distance’ will return the distances between neighbors according to the given metric'}),

                  ('metric', 'combo', {'items': ['euclidean', 'manhattan', 'chebyshev',
                                                 'minkowski', 'wminkowski', 'seuclidean', 'mahalanobis'],
                                       'toolTip': 'The distance metric used to calculate the k-Neighbors for each sample point.'
                                                  '\nThe DistanceMetric class gives a list of available metrics.\n'
                                                  'The default distance is ‘euclidean’ (‘minkowski’ metric with the p param equal to 2.)'}),

                  ('p', 'intSpin', {'min': 0, 'max': 100, 'value': 2, 'step': 1,
                                    'toolTip': 'Power parameter for the Minkowski metric.\n'
                                               'When p = 1, this is equivalent to using manhattan_distance (l1), and euclidean_distance (l2) for p = 2.\n'
                                               'For arbitrary p, minkowski_distance (l_p) is used.'}),

                  ('metric_params', 'kwargTextEdit',
                   {'placeHolder': 'additional keyword arguments for the metric function.'}),

                  ('include_self', 'check', {'toolTip': 'Mark each sample as the first nearest neighbor to itself.'}),

                  ('Apply', 'check', {'checked': False, 'applyBox': True})
                  ]

    def processData(self, transmission: Transmission):
        columns = transmission.df.columns
        self.ctrls['data_column'].setItems(columns.to_list())
        if self.ctrls['Apply'].isChecked() is False:
            return

        self.t = transmission.copy()

        n_neighbors = self.ctrls['n_neighbors'].value()
        mode = self.ctrls['mode'].currentText()
        metric = self.ctrls['metric'].currentText()
        p = self.ctrls['p'].value()
        metric_params = self.ctrls['metric_params'].get_kwargs()
        include_self = self.ctrls['include_self'].isChecked()

        output_column = '_KNEIGHBORS_GRAPH'

        data = self.t.df[self.data_column]

        kngraph = neighbors.kneighbors_graph(data, n_neighbors=n_neighbors, mode=mode, metric=metric,
                                             p=p, metric_params=metric_params, include_self=include_self,
                                             n_jobs=sys_cfg['HARDWARE']['n_processes'])
        kngraph_array = kngraph.toarray()

        self.t.df[output_column] = kngraph_array.tolist()

        params = {'data_column': self.data_column,
                  'n_neigbors': n_neighbors,
                  'mode': mode,
                  'metric': metric
                  }
        self.t.history_trace.add_operation(data_block_id='all', operation='kneighbors_graph', parameters=params)
        self.t.last_output = output_column

        return self.t



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file './main_window_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.12
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 600)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 30))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuModules = QtWidgets.QMenu(self.menubar)
        self.menuModules.setObjectName("menuModules")
        self.menuLoad_images = QtWidgets.QMenu(self.menuModules)
        self.menuLoad_images.setObjectName("menuLoad_images")
        self.menuFemtonics = QtWidgets.QMenu(self.menuLoad_images)
        self.menuFemtonics.setObjectName("menuFemtonics")
        self.menuCaImAn_toolbox = QtWidgets.QMenu(self.menuModules)
        self.menuCaImAn_toolbox.setObjectName("menuCaImAn_toolbox")
        self.menuCustom_Modules = QtWidgets.QMenu(self.menubar)
        self.menuCustom_Modules.setObjectName("menuCustom_Modules")
        self.menuImage = QtWidgets.QMenu(self.menubar)
        self.menuImage.setObjectName("menuImage")
        self.menuProjections = QtWidgets.QMenu(self.menuImage)
        self.menuProjections.setObjectName("menuProjections")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        self.menuHelp = QtWidgets.QMenu(self.menubar)
        self.menuHelp.setObjectName("menuHelp")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.dockConsole = QtWidgets.QDockWidget(MainWindow)
        self.dockConsole.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockConsole.setObjectName("dockConsole")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.dockConsole.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(8), self.dockConsole)
        self.actionOpen_work_environment = QtWidgets.QAction(MainWindow)
        self.actionOpen_work_environment.setObjectName("actionOpen_work_environment")
        self.actionSave_work_environment = QtWidgets.QAction(MainWindow)
        self.actionSave_work_environment.setObjectName("actionSave_work_environment")
        self.actionTiff_file = QtWidgets.QAction(MainWindow)
        self.actionTiff_file.setCheckable(False)
        self.actionTiff_file.setObjectName("actionTiff_file")
        self.actionMotion_Correction = QtWidgets.QAction(MainWindow)
        self.actionMotion_Correction.setCheckable(False)
        self.actionMotion_Correction.setObjectName("actionMotion_Correction")
        self.actionCNMF = QtWidgets.QAction(MainWindow)
        self.actionCNMF.setCheckable(False)
        self.actionCNMF.setObjectName("actionCNMF")
        self.actionROI_Manager = QtWidgets.QAction(MainWindow)
        self.actionROI_Manager.setCheckable(False)
        self.actionROI_Manager.setObjectName("actionROI_Manager")
        self.actionScript_Editor = QtWidgets.QAction(MainWindow)
        self.actionScript_Editor.setObjectName("actionScript_Editor")
        self.actionCNMF_E = QtWidgets.QAction(MainWindow)
        self.actionCNMF_E.setObjectName("actionCNMF_E")
        self.actionBatch_Manager = QtWidgets.QAction(MainWindow)
        self.actionBatch_Manager.setObjectName("actionBatch_Manager")
        self.actionDump_Work_Environment = QtWidgets.QAction(MainWindow)
        self.actionDump_Work_Environment.setObjectName("actionDump_Work_Environment")
        self.actionAdd_to_project = QtWidgets.QAction(MainWindow)
        self.actionAdd_to_project.setObjectName("actionAdd_to_project")
        self.actionResize = QtWidgets.QAction(MainWindow)
        self.actionResize.setObjectName("actionResize")
        self.actionCrop = QtWidgets.QAction(MainWindow)
        self.actionCrop.setObjectName("actionCrop")
        self.actionReset_Scale = QtWidgets.QAction(MainWindow)
        self.actionReset_Scale.setObjectName("actionReset_Scale")
        self.actionMeta_data = QtWidgets.QAction(MainWindow)
        self.actionMeta_data.setObjectName("actionMeta_data")
        self.actionMeasure = QtWidgets.QAction(MainWindow)
        self.actionMeasure.setObjectName("actionMeasure")
        self.actionStimulus_Mapping = QtWidgets.QAction(MainWindow)
        self.actionStimulus_Mapping.setObjectName("actionStimulus_Mapping")
        self.actionConsole = QtWidgets.QAction(MainWindow)
        self.actionConsole.setCheckable(True)
        self.actionConsole.setObjectName("actionConsole")
        self.actionWork_Environment_Info = QtWidgets.QAction(MainWindow)
        self.actionWork_Environment_Info.setObjectName("actionWork_Environment_Info")
        self.actionMean = QtWidgets.QAction(MainWindow)
        self.actionMean.setObjectName("actionMean")
        self.actionMax = QtWidgets.QAction(MainWindow)
        self.actionMax.setObjectName("actionMax")
        self.actionStandard_Deviation = QtWidgets.QAction(MainWindow)
        self.actionStandard_Deviation.setObjectName("actionStandard_Deviation")
        self.actionClose_all_projection_windows = QtWidgets.QAction(MainWindow)
        self.actionClose_all_projection_windows.setObjectName("actionClose_all_projection_windows")
        self.actionOpen_docs = QtWidgets.QAction(MainWindow)
        self.actionOpen_docs.setObjectName("actionOpen_docs")
        self.actionSuite2p_Importer = QtWidgets.QAction(MainWindow)
        self.actionSuite2p_Importer.setObjectName("actionSuite2p_Importer")
        self.actionCNMF_3D = QtWidgets.QAction(MainWindow)
        self.actionCNMF_3D.setObjectName("actionCNMF_3D")
        self.actionCaimanImportHDF5 = QtWidgets.QAction(MainWindow)
        self.actionCaimanImportHDF5.setObjectName("actionCaimanImportHDF5")
        self.actionVersion_0_2 = QtWidgets.QAction(MainWindow)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.actionVersion_0_2.setFont(font)
        self.actionVersion_0_2.setObjectName("actionVersion_0_2")
        self.actionCaimanDFOF = QtWidgets.QAction(MainWindow)
        self.actionCaimanDFOF.setObjectName("actionCaimanDFOF")
        self.actionReport_issue_bug = QtWidgets.QAction(MainWindow)
        self.actionReport_issue_bug.setObjectName("actionReport_issue_bug")
        self.actionQuestions_discussion = QtWidgets.QAction(MainWindow)
        self.actionQuestions_discussion.setObjectName("actionQuestions_discussion")
        self.action_mesc_importer = QtWidgets.QAction(MainWindow)
        self.action_mesc_importer.setObjectName("action_mesc_importer")
        self.action_mes_importer = QtWidgets.QAction(MainWindow)
        self.action_mes_importer.setObjectName("action_mes_importer")
        self.actionNuSeT_Segmentation = QtWidgets.QAction(MainWindow)
        self.actionNuSeT_Segmentation.setObjectName("actionNuSeT_Segmentation")
        self.actionInscopix_isxd = QtWidgets.QAction(MainWindow)
        self.actionInscopix_isxd.setObjectName("actionInscopix_isxd")
        self.menuFile.addAction(self.actionAdd_to_project)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionOpen_work_environment)
        self.menuFile.addAction(self.actionSave_work_environment)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionDump_Work_Environment)
        self.menuFemtonics.addAction(self.action_mesc_importer)
        self.menuFemtonics.addAction(self.action_mes_importer)
        self.menuLoad_images.addAction(self.actionTiff_file)
        self.menuLoad_images.addAction(self.menuFemtonics.menuAction())
        self.menuLoad_images.addAction(self.actionInscopix_isxd)
        self.menuCaImAn_toolbox.addAction(self.actionMotion_Correction)
        self.menuCaImAn_toolbox.addAction(self.actionCNMF)
        self.menuCaImAn_toolbox.addAction(self.actionCNMF_E)
        self.menuCaImAn_toolbox.addAction(self.actionCNMF_3D)
        self.menuCaImAn_toolbox.addAction(self.actionCaimanImportHDF5)
        self.menuCaImAn_toolbox.addAction(self.actionCaimanDFOF)
        self.menuModules.addAction(self.menuLoad_images.menuAction())
        self.menuModules.addAction(self.menuCaImAn_toolbox.menuAction())
        self.menuModules.addAction(self.actionROI_Manager)
        self.menuModules.addAction(self.actionNuSeT_Segmentation)
        self.menuModules.addAction(self.actionSuite2p_Importer)
        self.menuModules.addAction(self.actionStimulus_Mapping)
        self.menuModules.addAction(self.actionScript_Editor)
        self.menuModules.addSeparator()
        self.menuModules.addAction(self.actionBatch_Manager)
        self.menuModules.addSeparator()
        self.menuProjections.addAction(self.actionMean)
        self.menuProjections.addAction(self.actionMax)
        self.menuProjections.addAction(self.actionStandard_Deviation)
        self.menuProjections.addSeparator()
        self.menuProjections.addAction(self.actionClose_all_projection_windows)
        self.menuImage.addAction(self.actionReset_Scale)
        self.menuImage.addAction(self.actionResize)
        self.menuImage.addAction(self.actionCrop)
        self.menuImage.addSeparator()
        self.menuImage.addAction(self.actionMeasure)
        self.menuImage.addSeparator()
        self.menuImage.addAction(self.menuProjections.menuAction())
        self.menuView.addAction(self.actionWork_Environment_Info)
        self.menuView.addSeparator()
        self.menuView.addAction(self.actionConsole)
        self.menuHelp.addAction(self.actionOpen_docs)
        self.menuHelp.addSeparator()
        self.menuHelp.addAction(self.actionReport_issue_bug)
        self.menuHelp.addAction(self.actionQuestions_discussion)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuImage.menuAction())
        self.menubar.addAction(self.menuModules.menuAction())
        self.menubar.addAction(self.menuCustom_Modules.menuAction())
        self.menubar.addAction(self.menuView.menuAction())
        self.menubar.addAction(self.menuHelp.menuAction())

        self.retranslateUi(MainWindow)
        self.actionConsole.toggled['bool'].connect(self.dockConsole.setVisible)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.menuFile.setTitle(_translate("MainWindow", "&File"))
        self.menuModules.setTitle(_translate("MainWindow", "&Modules"))
        self.menuLoad_images.setTitle(_translate("MainWindow", "&Load images"))
        self.menuFemtonics.setTitle(_translate("MainWindow", "&Femtonics"))
        self.menuCaImAn_toolbox.setTitle(_translate("MainWindow", "&CaImAn toolbox"))
        self.menuCustom_Modules.setTitle(_translate("MainWindow", "&Plugins"))
        self.menuImage.setTitle(_translate("MainWindow", "&Image"))
        self.menuProjections.setTitle(_translate("MainWindow", "&Projections"))
        self.menuView.setTitle(_translate("MainWindow", "&View"))
        self.menuHelp.setTitle(_translate("MainWindow", "&Help"))
        self.dockConsole.setWindowTitle(_translate("MainWindow", "&Console: viewer"))
        self.actionOpen_work_environment.setText(_translate("MainWindow", "&Open work environment"))
        self.actionSave_work_environment.setText(_translate("MainWindow", "&Save work environment"))
        self.actionTiff_file.setText(_translate("MainWindow", "&Tiff file"))
        self.actionMotion_Correction.setText(_translate("MainWindow", "&Motion Correction"))
        self.actionCNMF.setText(_translate("MainWindow", "&CNMF"))
        self.actionROI_Manager.setText(_translate("MainWindow", "&ROI Manager"))
        self.actionROI_Manager.setShortcut(_translate("MainWindow", "Ctrl+R"))
        self.actionScript_Editor.setText(_translate("MainWindow", "Script &Editor"))
        self.actionCNMF_E.setText(_translate("MainWindow", "CNMF-&E"))
        self.actionBatch_Manager.setText(_translate("MainWindow", "&Batch Manager"))
        self.actionDump_Work_Environment.setText(_translate("MainWindow", "&Clear Work Environment"))
        self.actionAdd_to_project.setText(_translate("MainWindow", "&Add to project"))
        self.actionResize.setText(_translate("MainWindow", "Re&size"))
        self.actionCrop.setText(_translate("MainWindow", "&Crop"))
        self.actionReset_Scale.setText(_translate("MainWindow", "&Reset Scale"))
        self.actionMeta_data.setText(_translate("MainWindow", "&Meta data"))
        self.actionMeasure.setText(_translate("MainWindow", "&Measure Tool"))
        self.actionStimulus_Mapping.setText(_translate("MainWindow", "&Stimulus Mapping"))
        self.actionConsole.setText(_translate("MainWindow", "&Console"))
        self.actionWork_Environment_Info.setText(_translate("MainWindow", "&Work Environment Viewer"))
        self.actionMean.setText(_translate("MainWindow", "&Mean"))
        self.actionMax.setText(_translate("MainWindow", "Ma&x"))
        self.actionStandard_Deviation.setText(_translate("MainWindow", "&Standard Deviation"))
        self.actionClose_all_projection_windows.setText(_translate("MainWindow", "&Close all projection windows"))
        self.actionOpen_docs.setText(_translate("MainWindow", "&Open docs"))
        self.actionSuite2p_Importer.setText(_translate("MainWindow", "Suite&2p Importer"))
        self.actionCNMF_3D.setText(_translate("MainWindow", "CNMF &3D"))
        self.actionCaimanImportHDF5.setText(_translate("MainWindow", "&Import HDF5 result file"))
        self.actionVersion_0_2.setText(_translate("MainWindow", "&Version 0.2"))
        self.actionCaimanDFOF.setText(_translate("MainWindow", "&Detrend ΔF/F"))
        self.actionReport_issue_bug.setText(_translate("MainWindow", "&Report issue/bug"))
        self.actionQuestions_discussion.setText(_translate("MainWindow", "&Questions/discussion"))
        self.action_mesc_importer.setText(_translate("MainWindow", ".&mesc importer"))
        self.action_mes_importer.setText(_translate("MainWindow", ".mes &importer"))
        self.actionNuSeT_Segmentation.setText(_translate("MainWindow", "&NuSeT Segmentation"))
        self.actionInscopix_isxd.setText(_translate("MainWindow", "Inscopix isxd"))





# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'stats_gui_pytemplate.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1042, 709)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.gridLayout = QtWidgets.QGridLayout(self.centralwidget)
        self.gridLayout.setObjectName("gridLayout")
        self.tabWidget = QtWidgets.QTabWidget(self.centralwidget)
        self.tabWidget.setEnabled(True)
        self.tabWidget.setTabBarAutoHide(True)
        self.tabWidget.setObjectName("tabWidget")
        self.groups_tab = QtWidgets.QWidget()
        self.groups_tab.setObjectName("groups_tab")
        self.gridLayout_4 = QtWidgets.QGridLayout(self.groups_tab)
        self.gridLayout_4.setObjectName("gridLayout_4")
        self.label = QtWidgets.QLabel(self.groups_tab)
        self.label.setObjectName("label")
        self.gridLayout_4.addWidget(self.label, 0, 0, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.groups_tab)
        self.label_2.setObjectName("label_2")
        self.gridLayout_4.addWidget(self.label_2, 0, 1, 1, 1)
        self.listwGroups = QtWidgets.QListWidget(self.groups_tab)
        self.listwGroups.setDragDropMode(QtWidgets.QAbstractItemView.DragDrop)
        self.listwGroups.setObjectName("listwGroups")
        self.gridLayout_4.addWidget(self.listwGroups, 1, 0, 1, 1)
        self.listwStats = QtWidgets.QListWidget(self.groups_tab)
        self.listwStats.setDragDropMode(QtWidgets.QAbstractItemView.DragDrop)
        self.listwStats.setObjectName("listwStats")
        self.gridLayout_4.addWidget(self.listwStats, 1, 1, 1, 1)
        self.pushButton = QtWidgets.QPushButton(self.groups_tab)
        self.pushButton.setObjectName("pushButton")
        self.gridLayout_4.addWidget(self.pushButton, 0, 2, 1, 1)
        self.tabWidget.addTab(self.groups_tab, "")
        self.peak_plot_tab = QtWidgets.QWidget()
        self.peak_plot_tab.setObjectName("peak_plot_tab")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.peak_plot_tab)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.tabWidget.addTab(self.peak_plot_tab, "")
        self.stim_plots_tab = QtWidgets.QWidget()
        self.stim_plots_tab.setObjectName("stim_plots_tab")
        self.gridLayout_6 = QtWidgets.QGridLayout(self.stim_plots_tab)
        self.gridLayout_6.setObjectName("gridLayout_6")
        self.tabWidget.addTab(self.stim_plots_tab, "")
        self.violin_tab = QtWidgets.QWidget()
        self.violin_tab.setObjectName("violin_tab")
        self.gridLayout_3 = QtWidgets.QGridLayout(self.violin_tab)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.tabWidget.addTab(self.violin_tab, "")
        self.beeswarm_tab = QtWidgets.QWidget()
        self.beeswarm_tab.setObjectName("beeswarm_tab")
        self.gridLayout_8 = QtWidgets.QGridLayout(self.beeswarm_tab)
        self.gridLayout_8.setObjectName("gridLayout_8")
        self.tabWidget.addTab(self.beeswarm_tab, "")
        self.paracor_tab = QtWidgets.QWidget()
        self.paracor_tab.setObjectName("paracor_tab")
        self.gridLayout_7 = QtWidgets.QGridLayout(self.paracor_tab)
        self.gridLayout_7.setObjectName("gridLayout_7")
        self.tabWidget.addTab(self.paracor_tab, "")
        self.gridLayout.addWidget(self.tabWidget, 0, 0, 1, 1)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1042, 23))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        self.menuView = QtWidgets.QMenu(self.menubar)
        self.menuView.setObjectName("menuView")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.dockWidget = QtWidgets.QDockWidget(MainWindow)
        self.dockWidget.setMinimumSize(QtCore.QSize(250, 66))
        self.dockWidget.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockWidget.setWindowTitle("")
        self.dockWidget.setObjectName("dockWidget")
        self.dockWidgetContents = QtWidgets.QWidget()
        self.dockWidgetContents.setObjectName("dockWidgetContents")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.dockWidgetContents)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.stackedWidget = QtWidgets.QStackedWidget(self.dockWidgetContents)
        self.stackedWidget.setObjectName("stackedWidget")
        self.stack_page_transmission_history = QtWidgets.QWidget()
        self.stack_page_transmission_history.setObjectName("stack_page_transmission_history")
        self.stackedWidget.addWidget(self.stack_page_transmission_history)
        self.stack_page_curve_plots = QtWidgets.QWidget()
        self.stack_page_curve_plots.setObjectName("stack_page_curve_plots")
        self.stackedWidget.addWidget(self.stack_page_curve_plots)
        self.stack_page_stim_plots = QtWidgets.QWidget()
        self.stack_page_stim_plots.setObjectName("stack_page_stim_plots")
        self.stackedWidget.addWidget(self.stack_page_stim_plots)
        self.stack_page_violin_plots = QtWidgets.QWidget()
        self.stack_page_violin_plots.setObjectName("stack_page_violin_plots")
        self.stackedWidget.addWidget(self.stack_page_violin_plots)
        self.stack_page_beeswarm_plots = QtWidgets.QWidget()
        self.stack_page_beeswarm_plots.setObjectName("stack_page_beeswarm_plots")
        self.stackedWidget.addWidget(self.stack_page_beeswarm_plots)
        self.stack_page_paracor_plots = QtWidgets.QWidget()
        self.stack_page_paracor_plots.setObjectName("stack_page_paracor_plots")
        self.stackedWidget.addWidget(self.stack_page_paracor_plots)
        self.gridLayout_5.addWidget(self.stackedWidget, 0, 0, 1, 1)
        self.dockWidget.setWidget(self.dockWidgetContents)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(1), self.dockWidget)
        self.dockConsole = QtWidgets.QDockWidget(MainWindow)
        self.dockConsole.setMinimumSize(QtCore.QSize(59, 200))
        self.dockConsole.setFeatures(QtWidgets.QDockWidget.DockWidgetFloatable|QtWidgets.QDockWidget.DockWidgetMovable)
        self.dockConsole.setObjectName("dockConsole")
        self.dockWidgetContents_2 = QtWidgets.QWidget()
        self.dockWidgetContents_2.setObjectName("dockWidgetContents_2")
        self.dockConsole.setWidget(self.dockWidgetContents_2)
        MainWindow.addDockWidget(QtCore.Qt.DockWidgetArea(8), self.dockConsole)
        self.actionSave_Statistics_DataFrame = QtWidgets.QAction(MainWindow)
        self.actionSave_Statistics_DataFrame.setObjectName("actionSave_Statistics_DataFrame")
        self.actionSave_Group_Transmissions = QtWidgets.QAction(MainWindow)
        self.actionSave_Group_Transmissions.setObjectName("actionSave_Group_Transmissions")
        self.actionLoad_Groups = QtWidgets.QAction(MainWindow)
        self.actionLoad_Groups.setObjectName("actionLoad_Groups")
        self.actionConsole = QtWidgets.QAction(MainWindow)
        self.actionConsole.setCheckable(True)
        self.actionConsole.setObjectName("actionConsole")
        self.actionSave_Incoming_Transmissions = QtWidgets.QAction(MainWindow)
        self.actionSave_Incoming_Transmissions.setObjectName("actionSave_Incoming_Transmissions")
        self.actionOpen_Statistics_DataFrame = QtWidgets.QAction(MainWindow)
        self.actionOpen_Statistics_DataFrame.setObjectName("actionOpen_Statistics_DataFrame")
        self.actionControls_Dock = QtWidgets.QAction(MainWindow)
        self.actionControls_Dock.setCheckable(True)
        self.actionControls_Dock.setChecked(True)
        self.actionControls_Dock.setObjectName("actionControls_Dock")
        self.actionExport_Statistics_DataFrame = QtWidgets.QAction(MainWindow)
        self.actionExport_Statistics_DataFrame.setObjectName("actionExport_Statistics_DataFrame")
        self.actionExport_Groups = QtWidgets.QAction(MainWindow)
        self.actionExport_Groups.setObjectName("actionExport_Groups")
        self.menuFile.addAction(self.actionSave_Statistics_DataFrame)
        self.menuFile.addAction(self.actionOpen_Statistics_DataFrame)
        self.menuFile.addAction(self.actionExport_Statistics_DataFrame)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionSave_Group_Transmissions)
        self.menuFile.addAction(self.actionLoad_Groups)
        self.menuFile.addAction(self.actionExport_Groups)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionSave_Incoming_Transmissions)
        self.menuView.addAction(self.actionConsole)
        self.menuView.addAction(self.actionControls_Dock)
        self.menubar.addAction(self.menuFile.menuAction())
        self.menubar.addAction(self.menuView.menuAction())

        self.retranslateUi(MainWindow)
        self.tabWidget.setCurrentIndex(4)
        self.stackedWidget.setCurrentIndex(5)
        self.actionConsole.toggled['bool'].connect(self.dockConsole.setVisible)
        self.actionControls_Dock.toggled['bool'].connect(self.dockWidget.setVisible)
        self.tabWidget.currentChanged['int'].connect(self.stackedWidget.setCurrentIndex)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "MainWindow"))
        self.label.setText(_translate("MainWindow", "Open Groups"))
        self.label_2.setText(_translate("MainWindow", "Groups for Stats & Plots"))
        self.pushButton.setText(_translate("MainWindow", "Set"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.groups_tab), _translate("MainWindow", "Groups"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.peak_plot_tab), _translate("MainWindow", "Peak Plots"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.stim_plots_tab), _translate("MainWindow", "Stim Plots"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.violin_tab), _translate("MainWindow", "Violin Plots"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.beeswarm_tab), _translate("MainWindow", "Beeswarm Plots"))
        self.tabWidget.setTabText(self.tabWidget.indexOf(self.paracor_tab), _translate("MainWindow", "Parallel Coordinates"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.menuView.setTitle(_translate("MainWindow", "View"))
        self.dockConsole.setWindowTitle(_translate("MainWindow", "Statistics Console"))
        self.actionSave_Statistics_DataFrame.setText(_translate("MainWindow", "Save Statistics DataFrame"))
        self.actionSave_Group_Transmissions.setText(_translate("MainWindow", "Save Groups"))
        self.actionLoad_Groups.setText(_translate("MainWindow", "Load Groups"))
        self.actionConsole.setText(_translate("MainWindow", "Console"))
        self.actionSave_Incoming_Transmissions.setText(_translate("MainWindow", "Save Incoming Raw Transmissions"))
        self.actionOpen_Statistics_DataFrame.setText(_translate("MainWindow", "Open Statistics DataFrame"))
        self.actionControls_Dock.setText(_translate("MainWindow", "Controls Dock"))
        self.actionExport_Statistics_DataFrame.setText(_translate("MainWindow", "Export Statistics DataFrame"))
        self.actionExport_Groups.setText(_translate("MainWindow", "Export Groups"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'link_viewers_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.6
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(204, 320)
        self.listWidgetSource = QtWidgets.QListWidget(Form)
        self.listWidgetSource.setGeometry(QtCore.QRect(10, 30, 61, 221))
        self.listWidgetSource.setObjectName("listWidgetSource")
        self.label = QtWidgets.QLabel(Form)
        self.label.setGeometry(QtCore.QRect(20, 10, 41, 18))
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.label_2 = QtWidgets.QLabel(Form)
        self.label_2.setGeometry(QtCore.QRect(130, 10, 58, 18))
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.listWidgetReceivers = QtWidgets.QListWidget(Form)
        self.listWidgetReceivers.setGeometry(QtCore.QRect(130, 30, 61, 221))
        self.listWidgetReceivers.setSelectionMode(QtWidgets.QAbstractItemView.ExtendedSelection)
        self.listWidgetReceivers.setObjectName("listWidgetReceivers")
        self.label_3 = QtWidgets.QLabel(Form)
        self.label_3.setGeometry(QtCore.QRect(70, 100, 61, 71))
        font = QtGui.QFont()
        font.setPointSize(52)
        font.setBold(False)
        font.setWeight(50)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.btnSet = QtWidgets.QPushButton(Form)
        self.btnSet.setGeometry(QtCore.QRect(110, 280, 80, 26))
        self.btnSet.setObjectName("btnSet")
        self.btnDisconnect = QtWidgets.QPushButton(Form)
        self.btnDisconnect.setGeometry(QtCore.QRect(10, 280, 80, 26))
        self.btnDisconnect.setObjectName("btnDisconnect")

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label.setText(_translate("Form", "Source"))
        self.label_2.setText(_translate("Form", "Receivers"))
        self.label_3.setText(_translate("Form", "→"))
        self.btnSet.setText(_translate("Form", "Set"))
        self.btnDisconnect.setText(_translate("Form", "Disconnect"))




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on June 26 2018

@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007

"""

from ..core.common import ViewerUtils
from .stimmap_modules.page import Page
from .stimmap_modules.main_widget_pytemplate import *
from ...pyqtgraphCore.imageview import ImageView
from ...pyqtgraphCore import LinearRegionItem
from ...common import configuration, get_project_manager
import pandas as pd
from typing import *


class ModuleGUI(QtWidgets.QDockWidget):
    def __init__(self, parent, viewer):
        QtWidgets.QDockWidget.__init__(self, parent)
        self.vi = ViewerUtils(viewer)

        self.ui = Ui_MainWidget()
        self.ui.setupUi(self)
        # self.stim_types = dict.fromkeys(configuration.proj_cfg['STIM_DEFS'])
        self.tabs = {}

        if isinstance(self, ModuleGUI):
            self.setup_tabs()
            self.ui.comboBoxShowTimelineChoice.currentIndexChanged.connect(self.set_timeline)
            self.timeline_stimulus_display = TimeLineStimulusMap(viewer)
            get_project_manager().signal_project_config_changed.connect(self.reset)
            self.ui.btnSetAllMaps.clicked.connect(self.set_all_maps)

    @property
    def maps(self) -> dict:
        """Returns a dictionary of the stimulus maps"""
        return self.tabs

    def setup_tabs(self):
        self.ui.comboBoxShowTimelineChoice.addItems([''])
        self.ui.comboBoxShowTimelineChoice.addItems(configuration.proj_cfg.options('STIM_DEFS'))
        for stim_def in configuration.proj_cfg.options('STIM_DEFS'):
            self.add_stim_type(stim_def)

    def add_stim_type(self, stim_type: str):
        self.ui.tabWidget.addTab(Page(parent=self, stim_type=stim_type), stim_type)
        self.tabs[stim_type] = self.ui.tabWidget.widget(self.ui.tabWidget.count() - 1)

    def get_all_stims_dataframes(self) -> dict:
        d = {}
        for ix in range(self.ui.tabWidget.count()):
            stim_type = self.ui.tabWidget.widget(ix).stim_type
            try:
                df = self.ui.tabWidget.widget(ix).get_dataframe()
            except IndexError:
                d[stim_type] = None
            else:
                d[stim_type] = {}
                d[stim_type]['dataframe'] = df
                d[stim_type]['units'] = self.ui.tabWidget.widget(ix).ui.comboBoxTimeUnits.currentText()
        return d

    def export_map(self):
        path = QtWidgets.QFileDialog.getSaveFileName(None, 'Save stimulus map as', '', '(*.smap)')
        if path == '':
            return
        if path[0].endswith('.smap'):
            path = path[0]
        else:
            path = path[0] + '.smap'

        current_tab = self.ui.tabWidget.currentWidget()
        df = current_tab.get_dataframe()

        pd.to_pickle(df, path, protocol=4)

    def import_map(self):
        pass

    def clear_all_tabs(self):
        for stim_def in self.tabs.keys():
            tab = self.tabs[stim_def]
            assert isinstance(tab, Page)
            tab.clear()
        self.ui.tabWidget.clear()
        self.ui.comboBoxShowTimelineChoice.clear()

    def reset(self):
        self.clear_all_tabs()
        self.setup_tabs()

    def set_all_data(self, dataframes_dict: dict):
        self.reset()
        for stim_type in dataframes_dict.keys():
            if dataframes_dict[stim_type] is None:
                continue
                
            tab = self.tabs[stim_type]
            tab.set_data(dataframes_dict[stim_type])#['dataframe'])
            # if dataframes_dict[stim_type]['units'] == 'seconds':
            tab.ui.comboBoxTimeUnits.setCurrentIndex(0)
            # elif dataframes_dict[stim_type]['units'] == 'frames':
            #     tab.ui.comboBoxTimeUnits.setCurrentIndex(1)

    def set_all_maps(self):
        self.set_timeline(self.ui.comboBoxShowTimelineChoice.currentIndex())

    def export_to_work_env(self):
        d = self.get_all_stims_dataframes()

        for stim_type in d.keys():
            if d[stim_type] is None:
                continue

            if d[stim_type]['units'] == 'seconds':
                fps = self.vi.viewer.workEnv.meta['fps']
                d[stim_type]['dataframe']['start'] = d[stim_type]['dataframe']['start'] * fps
                d[stim_type]['dataframe']['start'] = d[stim_type]['dataframe']['start'].astype(int)

                d[stim_type]['dataframe']['end'] = d[stim_type]['dataframe']['end'] * fps
                d[stim_type]['dataframe']['end'] = d[stim_type]['dataframe']['end'].astype(int)

            d[stim_type] = d[stim_type]['dataframe']

        self.vi.viewer.workEnv.stim_maps = d

    def set_timeline(self, ix: int):
        self.export_to_work_env()
        if ix == 0:
            self.timeline_stimulus_display.clear_all()
            return

        stim_type = self.ui.comboBoxShowTimelineChoice.itemText(ix)
        if stim_type == '':
            self.timeline_stimulus_display.clear_all()
            return

        # tab_page = self.tabs[stim_type]
        # assert isinstance(tab_page, Page)

        try:
            # df = tab_page.get_dataframe()
            df = self.vi.viewer.workEnv.stim_maps[stim_type]
        except IndexError as ie:
            QtWidgets.QMessageBox.information(self, 'IndexError', str(ie))
            return

        # units = self.vi.viewer.workEnv.stim_maps[stim_type]['units']
        #
        # if units == 'seconds':
        #     fps = self.vi.viewer.workEnv.meta['fps']
        #     df['start'] = df['start'] * fps
        #     df['start'] = df['start'].astype(int)
        #     df['end'] = df['end'] * fps
        #     df['end'] = df['end'].astype(int)

        self.timeline_stimulus_display.set_from_stimulus_mapping(df)


class TimeLineStimulusMap:
    def __init__(self, viewer: ImageView):
        self.viewer = viewer
        self.linear_regions = []

    def add_linear_region(self, frame_start: int, frame_end: int, color: Tuple[float, float, float, float]):
        linear_region = LinearRegionItem(values=[frame_start, frame_end],
                                         brush=color, movable=False, bounds=[frame_start, frame_end])
        linear_region.setZValue(0)
        linear_region.lines[0].setPen(color)
        linear_region.lines[1].setPen(color)

        self.linear_regions.append(linear_region)
        self.viewer.ui.roiPlot.addItem(linear_region)

    def del_linear_region(self, linear_region: LinearRegionItem):
        self.viewer.ui.roiPlot.removeItem(linear_region)
        linear_region.deleteLater()
        self.linear_regions.remove(linear_region)

    def clear_all(self):
        for region in self.linear_regions:
            self.viewer.ui.roiPlot.removeItem(region)
            region.deleteLater()
        self.linear_regions = []

    def set_from_stimulus_mapping(self, dataframe: pd.DataFrame):
        """
        :param dataframe: dataframe of stimulus information with units as frames (not seconds)
                           for start and end times of stimuli
        """
        self.clear_all()
        for ix, row in dataframe.iterrows():
            self.add_linear_region(row['start'], row['end'], row['color'])


if __name__ == '__main__':
    app = QtWidgets.QApplication([])
    w = ModuleGUI(parent=None, viewer=None)
    w.add_stim_type('bah test')
    w.add_stim_type('bah test2')
    w.ui.tabWidget.clear()
    w.show()
    app.exec_()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
@author: kushal

Chatzigeorgiou Group
Sars International Centre for Marine Molecular Biology

GNU GENERAL PUBLIC LICENSE Version 3, 29 June 2007
"""

from ...pyqtgraphCore.widgets.MatplotlibWidget import MatplotlibWidget


class Lineplot(MatplotlibWidget):
    def __init__(self):
        MatplotlibWidget.__init__(self)
        self.ax = self.fig.add_subplot(111)
        self.plot = None

    def set(self):
        pass

    def set_multi(self, data, num_columns: int = 1):
        pass

    def clear(self):
        self.ax.cla()
